@@ -28,10 +28,10 @@
  *		   If the input is invalid, then \a *ctx remains unchanged.
  */
 css_error css__parse_azimuth(css_language *c,
-		const parserutils_vector *vector, int *ctx,
+		const parserutils_vector *vector, int32_t *ctx,
 		css_style *result)
 {
-	int orig_ctx = *ctx;
+	int32_t orig_ctx = *ctx;
 	css_error error;
 	const css_token *token;
 	uint8_t flags = 0;
@@ -53,10 +53,29 @@ css_error css__parse_azimuth(css_language *c,
 	}
 
 	if (token->type == CSS_TOKEN_IDENT &&
-		(lwc_string_caseless_isequal(token->idata, c->strings[INHERIT],
+			(lwc_string_caseless_isequal(
+			token->idata, c->strings[INHERIT],
 			&match) == lwc_error_ok && match)) {
 		parserutils_vector_iterate(vector, ctx);
 		flags = FLAG_INHERIT;
+	} else if (token->type == CSS_TOKEN_IDENT &&
+			(lwc_string_caseless_isequal(
+			token->idata, c->strings[INITIAL],
+			&match) == lwc_error_ok && match)) {
+		parserutils_vector_iterate(vector, ctx);
+		flags = FLAG_INITIAL;
+	} else if (token->type == CSS_TOKEN_IDENT &&
+			(lwc_string_caseless_isequal(
+			token->idata, c->strings[REVERT],
+			&match) == lwc_error_ok && match)) {
+		parserutils_vector_iterate(vector, ctx);
+		flags = FLAG_REVERT;
+	} else if (token->type == CSS_TOKEN_IDENT &&
+			(lwc_string_caseless_isequal(
+			token->idata, c->strings[UNSET],
+			&match) == lwc_error_ok && match)) {
+		parserutils_vector_iterate(vector, ctx);
+		flags = FLAG_UNSET;
 	} else if (token->type == CSS_TOKEN_IDENT &&
 		(lwc_string_caseless_isequal(token->idata, c->strings[LEFTWARDS],
 			&match) == lwc_error_ok && match)) {
