@@ -14,16 +14,16 @@
 #include "select/properties/properties.h"
 #include "select/properties/helpers.h"
 
-css_error css__cascade_border_bottom_width(uint32_t opv, css_style *style,
+css_error css__cascade_border_bottom_width(uint32_t opv, css_style *width,
 		css_select_state *state)
 {
-	return css__cascade_border_width(opv, style, state, set_border_bottom_width);
+	return css__cascade_border_width(opv, width, state, set_border_bottom_width);
 }
 
 css_error css__set_border_bottom_width_from_hint(const css_hint *hint,
-		css_computed_style *style)
+		css_computed_style *width)
 {
-	return set_border_bottom_width(style, hint->status,
+	return set_border_bottom_width(width, hint->status,
 			hint->data.length.value, hint->data.length.unit);
 }
 
@@ -33,6 +33,21 @@ css_error css__initial_border_bottom_width(css_select_state *state)
 			0, CSS_UNIT_PX);
 }
 
+css_error css__copy_border_bottom_width(
+		const css_computed_style *from,
+		css_computed_style *to)
+{
+	css_fixed length = 0;
+	css_unit unit = CSS_UNIT_PX;
+	uint8_t type = get_border_bottom_width(from, &length, &unit);
+
+	if (from == to) {
+		return CSS_OK;
+	}
+
+	return set_border_bottom_width(to, type, length, unit);
+}
+
 css_error css__compose_border_bottom_width(const css_computed_style *parent,
 		const css_computed_style *child,
 		css_computed_style *result)
@@ -41,10 +56,8 @@ css_error css__compose_border_bottom_width(const css_computed_style *parent,
 	css_unit unit = CSS_UNIT_PX;
 	uint8_t type = get_border_bottom_width(child, &length, &unit);
 
-	if (type == CSS_BORDER_WIDTH_INHERIT) {
-		type = get_border_bottom_width(parent, &length, &unit);
-	}
-
-	return set_border_bottom_width(result, type, length, unit);
+	return css__copy_border_bottom_width(
+			type == CSS_BORDER_WIDTH_INHERIT ? parent : child,
+			result);
 }
 
