@@ -21,7 +21,7 @@ css_error css__cascade_direction(uint32_t opv, css_style *style,
 
 	UNUSED(style);
 
-	if (isInherit(opv) == false) {
+	if (hasFlagValue(opv) == false) {
 		switch (getValue(opv)) {
 		case DIRECTION_LTR:
 			value = CSS_DIRECTION_LTR;
@@ -33,7 +33,7 @@ css_error css__cascade_direction(uint32_t opv, css_style *style,
 	}
 
 	if (css__outranks_existing(getOpcode(opv), isImportant(opv), state,
-			isInherit(opv))) {
+			getFlagValue(opv))) {
 		return set_direction(state->computed, value);
 	}
 
@@ -51,16 +51,25 @@ css_error css__initial_direction(css_select_state *state)
 	return set_direction(state->computed, CSS_DIRECTION_LTR);
 }
 
+css_error css__copy_direction(
+		const css_computed_style *from,
+		css_computed_style *to)
+{
+	if (from == to) {
+		return CSS_OK;
+	}
+
+	return set_direction(to, get_direction(from));
+}
+
 css_error css__compose_direction(const css_computed_style *parent,
 		const css_computed_style *child,
 		css_computed_style *result)
 {
 	uint8_t type = get_direction(child);
 
-	if (type == CSS_DIRECTION_INHERIT) {
-		type = get_direction(parent);
-	}
-
-	return set_direction(result, type);
+	return css__copy_direction(
+			type == CSS_DIRECTION_INHERIT ? parent : child,
+			result);
 }
 
