@@ -32,6 +32,21 @@ css_error css__initial_height(css_select_state *state)
 	return set_height(state->computed, CSS_HEIGHT_AUTO, 0, CSS_UNIT_PX);
 }
 
+css_error css__copy_height(
+		const css_computed_style *from,
+		css_computed_style *to)
+{
+	css_fixed length = 0;
+	css_unit unit = CSS_UNIT_PX;
+	uint8_t type = get_height(from, &length, &unit);
+
+	if (from == to) {
+		return CSS_OK;
+	}
+
+	return set_height(to, type, length, unit);
+}
+
 css_error css__compose_height(const css_computed_style *parent,
 		const css_computed_style *child,
 		css_computed_style *result)
@@ -40,10 +55,8 @@ css_error css__compose_height(const css_computed_style *parent,
 	css_unit unit = CSS_UNIT_PX;
 	uint8_t type = get_height(child, &length, &unit);
 
-	if (type == CSS_HEIGHT_INHERIT) {
-		type = get_height(parent, &length, &unit);
-	}
-
-	return set_height(result, type, length, unit);
+	return css__copy_height(
+			type == CSS_HEIGHT_INHERIT ? parent : child,
+			result);
 }
 
