@@ -33,6 +33,21 @@ css_error css__initial_max_height(css_select_state *state)
 			0, CSS_UNIT_PX);
 }
 
+css_error css__copy_max_height(
+		const css_computed_style *from,
+		css_computed_style *to)
+{
+	css_fixed length = 0;
+	css_unit unit = CSS_UNIT_PX;
+	uint8_t type = get_max_height(from, &length, &unit);
+
+	if (from == to) {
+		return CSS_OK;
+	}
+
+	return set_max_height(to, type, length, unit);
+}
+
 css_error css__compose_max_height(const css_computed_style *parent,
 		const css_computed_style *child,
 		css_computed_style *result)
@@ -41,10 +56,8 @@ css_error css__compose_max_height(const css_computed_style *parent,
 	css_unit unit = CSS_UNIT_PX;
 	uint8_t type = get_max_height(child, &length, &unit);
 
-	if (type == CSS_MAX_HEIGHT_INHERIT) {
-		type = get_max_height(parent, &length, &unit);
-	}
-
-	return set_max_height(result, type, length, unit);
+	return css__copy_max_height(
+			type == CSS_MAX_HEIGHT_INHERIT ? parent : child,
+			result);
 }
 
