@@ -33,6 +33,21 @@ css_error css__initial_outline_width(css_select_state *state)
 			0, CSS_UNIT_PX);
 }
 
+css_error css__copy_outline_width(
+		const css_computed_style *from,
+		css_computed_style *to)
+{
+	css_fixed length = 0;
+	css_unit unit = CSS_UNIT_PX;
+	uint8_t type = get_outline_width(from, &length, &unit);
+
+	if (from == to) {
+		return CSS_OK;
+	}
+
+	return set_outline_width(to, type, length, unit);
+}
+
 css_error css__compose_outline_width(const css_computed_style *parent,
 		const css_computed_style *child,
 		css_computed_style *result)
@@ -41,10 +56,8 @@ css_error css__compose_outline_width(const css_computed_style *parent,
 	css_unit unit = CSS_UNIT_PX;
 	uint8_t type = get_outline_width(child, &length, &unit);
 
-	if (type == CSS_OUTLINE_WIDTH_INHERIT) {
-		type = get_outline_width(parent, &length, &unit);
-	}
-
-	return set_outline_width(result, type, length, unit);
+	return css__copy_outline_width(
+			type == CSS_OUTLINE_WIDTH_INHERIT ? parent : child,
+			result);
 }
 
