@@ -33,16 +33,25 @@ css_error css__initial_page_break_inside(css_select_state *state)
 			CSS_PAGE_BREAK_INSIDE_AUTO);
 }
 
+css_error css__copy_page_break_inside(
+		const css_computed_style *from,
+		css_computed_style *to)
+{
+	if (from == to) {
+		return CSS_OK;
+	}
+
+	return set_page_break_inside(to, get_page_break_inside(from));
+}
+
 css_error css__compose_page_break_inside(const css_computed_style *parent,
 		const css_computed_style *child,
 		css_computed_style *result)
 {
 	uint8_t type = get_page_break_inside(child);
 
-	if (type == CSS_PAGE_BREAK_INSIDE_INHERIT) {
-		type = get_page_break_inside(parent);
-	}
-
-	return set_page_break_inside(result, type);
+	return css__copy_page_break_inside(
+			type == CSS_PAGE_BREAK_INSIDE_INHERIT ? parent : child,
+			result);
 }
 
