@@ -20,7 +20,7 @@ css_error css__cascade_pitch(uint32_t opv, css_style *style,
 	css_fixed freq = 0;
 	uint32_t unit = UNIT_HZ;
 
-	if (isInherit(opv) == false) {
+	if (hasFlagValue(opv) == false) {
 		switch (getValue(opv)) {
 		case PITCH_FREQUENCY:
 			freq = *((css_fixed *) style->bytecode);
@@ -35,13 +35,20 @@ css_error css__cascade_pitch(uint32_t opv, css_style *style,
 		case PITCH_X_HIGH:
 			/** \todo convert to public values */
 			break;
+		case PITCH_CALC:
+			advance_bytecode(style, sizeof(unit));
+			advance_bytecode(style, sizeof(unit)); // TODO
+			return CSS_OK;
+		default:
+			assert(0 && "Invalid value");
+			break;
 		}
 	}
 
 	unit = css__to_css_unit(unit);
 
 	if (css__outranks_existing(getOpcode(opv), isImportant(opv), state,
-			isInherit(opv))) {
+			getFlagValue(opv))) {
 		/** \todo pitch */
 	}
 
@@ -64,6 +71,16 @@ css_error css__initial_pitch(css_select_state *state)
 	return CSS_OK;
 }
 
+css_error css__copy_pitch(
+		const css_computed_style *from,
+		css_computed_style *to)
+{
+	UNUSED(from);
+	UNUSED(to);
+
+	return CSS_OK;
+}
+
 css_error css__compose_pitch(const css_computed_style *parent,
 		const css_computed_style *child,
 		css_computed_style *result)
