@@ -19,7 +19,7 @@ css_error css__cascade_play_during(uint32_t opv, css_style *style,
 {
 	lwc_string *uri = NULL;
 
-	if (isInherit(opv) == false) {
+	if (hasFlagValue(opv) == false) {
 		switch (getValue(opv)) {
 		case PLAY_DURING_URI:
 			css__stylesheet_string_get(style->sheet, *((css_code_t *) style->bytecode), &uri);
@@ -35,7 +35,7 @@ css_error css__cascade_play_during(uint32_t opv, css_style *style,
 	}
 
 	if (css__outranks_existing(getOpcode(opv), isImportant(opv), state,
-			isInherit(opv))) {
+			getFlagValue(opv))) {
 		/** \todo play-during */
 	}
 
@@ -58,6 +58,16 @@ css_error css__initial_play_during(css_select_state *state)
 	return CSS_OK;
 }
 
+css_error css__copy_play_during(
+		const css_computed_style *from,
+		css_computed_style *to)
+{
+	UNUSED(from);
+	UNUSED(to);
+
+	return CSS_OK;
+}
+
 css_error css__compose_play_during(const css_computed_style *parent,
 		const css_computed_style *child,
 		css_computed_style *result)
