@@ -14,12 +14,12 @@
 #include "select/properties/properties.h"
 #include "select/properties/helpers.h"
 
-css_error css__cascade_speak(uint32_t opv, css_style *style, 
+css_error css__cascade_speak(uint32_t opv, css_style *style,
 		css_select_state *state)
 {
 	UNUSED(style);
 
-	if (isInherit(opv) == false) {
+	if (hasFlagValue(opv) == false) {
 		switch (getValue(opv)) {
 		case SPEAK_NORMAL:
 		case SPEAK_NONE:
@@ -30,7 +30,7 @@ css_error css__cascade_speak(uint32_t opv, css_style *style,
 	}
 
 	if (css__outranks_existing(getOpcode(opv), isImportant(opv), state,
-			isInherit(opv))) {
+			getFlagValue(opv))) {
 		/** \todo speak */
 	}
 
@@ -53,6 +53,16 @@ css_error css__initial_speak(css_select_state *state)
 	return CSS_OK;
 }
 
+css_error css__copy_speak(
+		const css_computed_style *from,
+		css_computed_style *to)
+{
+	UNUSED(from);
+	UNUSED(to);
+
+	return CSS_OK;
+}
+
 css_error css__compose_speak(const css_computed_style *parent,
 		const css_computed_style *child,
 		css_computed_style *result)
