@@ -14,12 +14,12 @@
 #include "select/properties/properties.h"
 #include "select/properties/helpers.h"
 
-css_error css__cascade_speak_punctuation( 
+css_error css__cascade_speak_punctuation(
 		uint32_t opv, css_style *style, css_select_state *state)
 {
 	UNUSED(style);
 
-	if (isInherit(opv) == false) {
+	if (hasFlagValue(opv) == false) {
 		switch (getValue(opv)) {
 		case SPEAK_PUNCTUATION_CODE:
 		case SPEAK_PUNCTUATION_NONE:
@@ -29,7 +29,7 @@ css_error css__cascade_speak_punctuation(
 	}
 
 	if (css__outranks_existing(getOpcode(opv), isImportant(opv), state,
-			isInherit(opv))) {
+			getFlagValue(opv))) {
 		/** \todo speak-punctuation */
 	}
 
@@ -52,6 +52,16 @@ css_error css__initial_speak_punctuation(css_select_state *state)
 	return CSS_OK;
 }
 
+css_error css__copy_speak_punctuation(
+		const css_computed_style *from,
+		css_computed_style *to)
+{
+	UNUSED(from);
+	UNUSED(to);
+
+	return CSS_OK;
+}
+
 css_error css__compose_speak_punctuation(const css_computed_style *parent,
 		const css_computed_style *child,
 		css_computed_style *result)
