@@ -19,7 +19,7 @@ css_error css__cascade_speech_rate(uint32_t opv, css_style *style,
 {
 	css_fixed rate = 0;
 
-	if (isInherit(opv) == false) {
+	if (hasFlagValue(opv) == false) {
 		switch (getValue(opv)) {
 		case SPEECH_RATE_SET:
 			rate = *((css_fixed *) style->bytecode);
@@ -34,11 +34,18 @@ css_error css__cascade_speech_rate(uint32_t opv, css_style *style,
 		case SPEECH_RATE_SLOWER:
 			/** \todo convert to public values */
 			break;
+		case SPEECH_RATE_CALC:
+			advance_bytecode(style, sizeof(unit));
+			advance_bytecode(style, sizeof(unit)); // TODO
+			return CSS_OK;
+		default:
+			assert(0 && "Invalid value");
+			break;
 		}
 	}
 
 	if (css__outranks_existing(getOpcode(opv), isImportant(opv), state,
-			isInherit(opv))) {
+			getFlagValue(opv))) {
 		/** \todo speech-rate */
 	}
 
@@ -61,6 +68,16 @@ css_error css__initial_speech_rate(css_select_state *state)
 	return CSS_OK;
 }
 
+css_error css__copy_speech_rate(
+		const css_computed_style *from,
+		css_computed_style *to)
+{
+	UNUSED(from);
+	UNUSED(to);
+
+	return CSS_OK;
+}
+
 css_error css__compose_speech_rate(const css_computed_style *parent,
 		const css_computed_style *child,
 		css_computed_style *result)
