@@ -131,7 +131,9 @@ void dump_rule_media(css_rule_media *s, char **buf, size_t *buflen)
 	char *ptr = *buf;
 	css_rule *rule;
 
-	ptr += sprintf(ptr, "| @media ");
+	ptr += sprintf(ptr, "| @media %s%03lx",
+			s->media->negate_type ? "not " : "",
+			s->media->type);
 
 	/* \todo media list */
 
@@ -489,6 +491,8 @@ static const char *opcode_names[] = {
 	"flex-wrap",
 	"justify-content",
 	"order",
+	"fill-opacity",
+	"stroke-opacity",
 };
 
 static void dump_css_fixed(css_fixed f, char **ptr)
@@ -584,24 +588,15 @@ static void dump_unit(css_fixed val, uint32_t unit, char **ptr)
 	case UNIT_PC:
 		*ptr += sprintf(*ptr, "pc");
 		break;
-	case UNIT_CAP:
-		*ptr += sprintf(*ptr, "cap");
-		break;
 	case UNIT_CH:
 		*ptr += sprintf(*ptr, "ch");
 		break;
-	case UNIT_IC:
-		*ptr += sprintf(*ptr, "ic");
-		break;
 	case UNIT_REM:
 		*ptr += sprintf(*ptr, "rem");
 		break;
 	case UNIT_LH:
 		*ptr += sprintf(*ptr, "lh");
 		break;
-	case UNIT_RLH:
-		*ptr += sprintf(*ptr, "rlh");
-		break;
 	case UNIT_VH:
 		*ptr += sprintf(*ptr, "vh");
 		break;
@@ -795,8 +790,14 @@ void dump_bytecode(css_style *style, char **ptr, uint32_t depth)
 			*((*ptr)++) = ' ';
 		*ptr += sprintf(*ptr, "%s: ", opcode_names[op]);
 
-		if (isInherit(opv)) {
+		if (getFlagValue(opv) == FLAG_VALUE_INHERIT) {
 			*ptr += sprintf(*ptr, "inherit");
+		} else if (getFlagValue(opv) == FLAG_VALUE_INITIAL) {
+			*ptr += sprintf(*ptr, "initial");
+		} else if (getFlagValue(opv) == FLAG_VALUE_REVERT) {
+			*ptr += sprintf(*ptr, "revert");
+		} else if (getFlagValue(opv) == FLAG_VALUE_UNSET) {
+			*ptr += sprintf(*ptr, "unset");
 		} else {
 			value = getValue(opv);
 
@@ -1820,6 +1821,28 @@ void dump_bytecode(css_style *style, char **ptr, uint32_t depth)
 					break;
 				}
 				break;
+			case CSS_PROP_FILL_OPACITY:
+				switch (value) {
+				case FILL_OPACITY_SET:
+				{
+					css_fixed val = *((css_fixed *) bytecode);
+					ADVANCE(sizeof(val));
+					dump_number(val, ptr);
+				}
+					break;
+				}
+				break;
+			case CSS_PROP_STROKE_OPACITY:
+				switch (value) {
+				case STROKE_OPACITY_SET:
+				{
+					css_fixed val = *((css_fixed *) bytecode);
+					ADVANCE(sizeof(val));
+					dump_number(val, ptr);
+				}
+					break;
+				}
+				break;
 			case CSS_PROP_FLEX_BASIS:
 				switch (value) {
 				case FLEX_BASIS_AUTO:
