@@ -105,24 +105,15 @@ static size_t dump_css_unit(css_fixed val, css_unit unit, char *ptr, size_t len)
 	case CSS_UNIT_PC:
 		ret += snprintf(ptr + ret, len - ret, "pc");
 		break;
-	case CSS_UNIT_CAP:
-		ret += snprintf(ptr + ret, len - ret, "cap");
-		break;
 	case CSS_UNIT_CH:
 		ret += snprintf(ptr + ret, len - ret, "ch");
 		break;
-	case CSS_UNIT_IC:
-		ret += snprintf(ptr + ret, len - ret, "ic");
-		break;
 	case CSS_UNIT_REM:
 		ret += snprintf(ptr + ret, len - ret, "rem");
 		break;
 	case CSS_UNIT_LH:
 		ret += snprintf(ptr + ret, len - ret, "lh");
 		break;
-	case CSS_UNIT_RLH:
-		ret += snprintf(ptr + ret, len - ret, "rlh");
-		break;
 	case CSS_UNIT_VH:
 		ret += snprintf(ptr + ret, len - ret, "vh");
 		break;
@@ -1612,6 +1603,12 @@ static void dump_computed_style(const css_computed_style *style, char *buf,
 	case CSS_DISPLAY_INLINE_FLEX:
 		wrote = snprintf(ptr, *len, "display: inline-flex\n");
 		break;
+	case CSS_DISPLAY_GRID:
+		wrote = snprintf(ptr, *len, "display: grid\n");
+		break;
+	case CSS_DISPLAY_INLINE_GRID:
+		wrote = snprintf(ptr, *len, "display: inline-grid\n");
+		break;
 	default:
 		wrote = 0;
 		break;
@@ -1638,6 +1635,30 @@ static void dump_computed_style(const css_computed_style *style, char *buf,
 	ptr += wrote;
 	*len -= wrote;
 
+	/* fill-opacity */
+	val = css_computed_fill_opacity(style, &len1);
+	switch (val) {
+	case CSS_FILL_OPACITY_INHERIT:
+		wrote = snprintf(ptr, *len, "fill-opacity: inherit\n");
+		break;
+	case CSS_FILL_OPACITY_SET:
+		wrote = snprintf(ptr, *len, "fill-opacity: ");
+		ptr += wrote;
+		*len -= wrote;
+
+		wrote = dump_css_fixed(len1, ptr, *len);
+		ptr += wrote;
+		*len -= wrote;
+
+		wrote = snprintf(ptr, *len, "\n");
+		break;
+	default:
+		wrote = 0;
+		break;
+	}
+	ptr += wrote;
+	*len -= wrote;
+
 	/* flex-basis */
 	val = css_computed_flex_basis(style, &len1, &unit1);
 	switch (val) {
@@ -2728,6 +2749,9 @@ static void dump_computed_style(const css_computed_style *style, char *buf,
 	case CSS_POSITION_FIXED:
 		wrote = snprintf(ptr, *len, "position: fixed\n");
 		break;
+	case CSS_POSITION_STICKY:
+		wrote = snprintf(ptr, *len, "position: sticky\n");
+		break;
 	default:
 		wrote = 0;
 		break;
@@ -2796,6 +2820,30 @@ static void dump_computed_style(const css_computed_style *style, char *buf,
 	ptr += wrote;
 	*len -= wrote;
 
+	/* stroke-opacity */
+	val = css_computed_stroke_opacity(style, &len1);
+	switch (val) {
+	case CSS_STROKE_OPACITY_INHERIT:
+		wrote = snprintf(ptr, *len, "stroke-opacity: inherit\n");
+		break;
+	case CSS_STROKE_OPACITY_SET:
+		wrote = snprintf(ptr, *len, "stroke-opacity: ");
+		ptr += wrote;
+		*len -= wrote;
+
+		wrote = dump_css_fixed(len1, ptr, *len);
+		ptr += wrote;
+		*len -= wrote;
+
+		wrote = snprintf(ptr, *len, "\n");
+		break;
+	default:
+		wrote = 0;
+		break;
+	}
+	ptr += wrote;
+	*len -= wrote;
+
 	/* table-layout */
 	val = css_computed_table_layout(style);
 	switch (val) {
