@@ -49,7 +49,7 @@ struct dom_attr {
 };
 
 /* The vtable for dom_attr node */
-static struct dom_attr_vtable attr_vtable = {
+static const struct dom_attr_vtable attr_vtable = {
 	{
 		{
 			DOM_NODE_EVENT_TARGET_VTABLE,
@@ -60,7 +60,7 @@ static struct dom_attr_vtable attr_vtable = {
 };
 
 /* The protected vtable for dom_attr */
-static struct dom_node_protect_vtable attr_protect_vtable = {
+static const struct dom_node_protect_vtable attr_protect_vtable = {
 	DOM_ATTR_PROTECT_VTABLE
 };
 
@@ -221,7 +221,8 @@ dom_exception dom_attr_get_integer(dom_attr *a, uint32_t *value)
  * \param value  The new value
  * \return DOM_NO_ERR on success,
  *         DOM_ATTR_WRONG_TYPE_ERR if the attribute node is not a integer
- *                                 attribute
+ *                                 attribute,
+ *         DOM_NO_MODIFICATION_ALLOWED_ERR if ::a is readonly
  */
 dom_exception dom_attr_set_integer(dom_attr *a, uint32_t value)
 {
@@ -230,6 +231,10 @@ dom_exception dom_attr_set_integer(dom_attr *a, uint32_t value)
 	bool success = true;
 	dom_exception err;
 
+	if (_dom_node_readonly(&a->base)) {
+		return DOM_NO_MODIFICATION_ALLOWED_ERR;
+	}
+
 	/* If this is the first set method, we should fix this attribute
 	 * type */
 	if (a->type == DOM_ATTR_UNSET)
@@ -283,7 +288,8 @@ dom_exception dom_attr_get_short(dom_attr *a, unsigned short *value)
  * \param value  The new value
  * \return DOM_NO_ERR on success,
  *         DOM_ATTR_WRONG_TYPE_ERR if the attribute node is not a short
- *                                 attribute
+ *                                 attribute,
+ *         DOM_NO_MODIFICATION_ALLOWED_ERR if ::a is readonly
  */
 dom_exception dom_attr_set_short(dom_attr *a, unsigned short value)
 {
@@ -292,6 +298,10 @@ dom_exception dom_attr_set_short(dom_attr *a, unsigned short value)
 	bool success = true;
 	dom_exception err;
 
+	if (_dom_node_readonly(&a->base)) {
+		return DOM_NO_MODIFICATION_ALLOWED_ERR;
+	}
+
 	/* If this is the first set method, we should fix this attribute
 	 * type */
 	if (a->type == DOM_ATTR_UNSET)
@@ -345,7 +355,8 @@ dom_exception dom_attr_get_bool(dom_attr *a, bool *value)
  * \param value  The new value
  * \return DOM_NO_ERR on success,
  *         DOM_ATTR_WRONG_TYPE_ERR if the attribute node is not a bool
- *                                 attribute
+ *                                 attribute,
+ *         DOM_NO_MODIFICATION_ALLOWED_ERR if ::a is readonly
  */
 dom_exception dom_attr_set_bool(dom_attr *a, bool value)
 {
@@ -354,6 +365,10 @@ dom_exception dom_attr_set_bool(dom_attr *a, bool value)
 	bool success = true;
 	dom_exception err;
 
+	if (_dom_node_readonly(&a->base)) {
+		return DOM_NO_MODIFICATION_ALLOWED_ERR;
+	}
+
 	/* If this is the first set method, we should fix this attribute
 	 * type */
 	if (a->type == DOM_ATTR_UNSET)
