@@ -20,7 +20,7 @@
 #include "utils/utils.h"
 
 /* The virtual table for dom_text */
-struct dom_text_vtable text_vtable = {
+const struct dom_text_vtable text_vtable = {
 	{
 		{
 			{
@@ -33,7 +33,7 @@ struct dom_text_vtable text_vtable = {
 	DOM_TEXT_VTABLE
 };
 
-static struct dom_node_protect_vtable text_protect_vtable = {
+static const struct dom_node_protect_vtable text_protect_vtable = {
 	DOM_TEXT_PROTECT_VTABLE
 };
 
@@ -279,6 +279,7 @@ dom_exception _dom_text_replace_whole_text(dom_text *text,
 	err = walk_logic_adjacent_text(text, DELETE, &ret);
 	if (err != DOM_NO_ERR)
 		return err;
+	assert(ret == NULL);
 	
 	err = dom_characterdata_set_data(text, content);
 	if (err != DOM_NO_ERR)
