@@ -11,7 +11,7 @@
 
 static void _virtual_dom_ui_event_destroy(struct dom_event *evt);
 
-static struct dom_event_private_vtable _event_vtable = {
+static const struct dom_event_private_vtable _event_vtable = {
 	_virtual_dom_ui_event_destroy
 };
 
