@@ -72,7 +72,7 @@
 #include "utils/namespace.h"
 #include "utils/utils.h"
 
-static struct dom_html_document_vtable html_document_vtable = {
+static const struct dom_html_document_vtable html_document_vtable = {
 	{
 		{
 			{
@@ -85,7 +85,7 @@ static struct dom_html_document_vtable html_document_vtable = {
 	DOM_HTML_DOCUMENT_VTABLE
 };
 
-static struct dom_node_protect_vtable html_document_protect_vtable = {
+static const struct dom_node_protect_vtable html_document_protect_vtable = {
 	DOM_HTML_DOCUMENT_PROTECT_VTABLE
 };
 
@@ -134,13 +134,12 @@ dom_exception _dom_html_document_initialise(dom_html_document *doc,
 	doc->cookie = NULL;
 	doc->body = NULL;
 
-	doc->memoised = calloc(sizeof(dom_string *), hds_COUNT);
+	doc->memoised = calloc(hds_COUNT, sizeof(dom_string *));
 	if (doc->memoised == NULL) {
 		error = DOM_NO_MEM_ERR;
 		goto out;
 	}
-	doc->elements = calloc(sizeof(dom_string *),
-			DOM_HTML_ELEMENT_TYPE__COUNT);
+	doc->elements = calloc(DOM_HTML_ELEMENT_TYPE__COUNT, sizeof(dom_string *));
 	if (doc->elements == NULL) {
 		error = DOM_NO_MEM_ERR;
 		goto out;
@@ -319,6 +318,8 @@ static inline dom_html_element_type _dom_html_document_get_element_type(
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_RP)
 		else
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_RT)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TT)
 		break;
 	case 3:
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_DIV)
@@ -354,6 +355,8 @@ static inline dom_html_element_type _dom_html_document_get_element_type(
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_BDO)
 		else
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_WBR)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_BIG)
 		break;
 	case 4:
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_META)
@@ -395,6 +398,8 @@ static inline dom_html_element_type _dom_html_document_get_element_type(
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_BASE)
 		else
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TIME)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_NOBR)
 		break;
 	case 5:
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_INPUT)
@@ -469,6 +474,10 @@ static inline dom_html_element_type _dom_html_document_get_element_type(
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_SOURCE)
 		else
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_STRONG)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_SPACER)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_STRIKE)
 		break;
 	case 7:
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_CAPTION)
@@ -486,6 +495,12 @@ static inline dom_html_element_type _dom_html_document_get_element_type(
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_SECTION)
 		else
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_SUMMARY)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_ACRONYM)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_BGSOUND)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_MARQUEE)
 		break;
 	case 8:
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TEXTAREA)
@@ -509,6 +524,8 @@ static inline dom_html_element_type _dom_html_document_get_element_type(
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_PROGRESS)
 		else
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_TEMPLATE)
+		else
+		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_NOFRAMES)
 		break;
 	case 10:
 		RETURN_IF_MATCH(upper, DOM_HTML_ELEMENT_TYPE_BLOCKQUOTE)
@@ -536,6 +553,10 @@ _dom_html_document_create_element_internal(
 	dom_exception exc;
 	struct dom_html_element_create_params params;
 
+	/* If the input tag name is empty, this is an 'invalid character' error */
+	if (dom_string_length(in_tag_name) == 0)
+		return DOM_INVALID_CHARACTER_ERR;
+
 	exc = dom_string_toupper(in_tag_name, true, &params.name);
 	if (exc != DOM_NO_ERR)
 		return exc;
@@ -805,8 +826,10 @@ _dom_html_document_create_element_internal(
 	case DOM_HTML_ELEMENT_TYPE_EM:
 	case DOM_HTML_ELEMENT_TYPE_RP:
 	case DOM_HTML_ELEMENT_TYPE_RT:
+	case DOM_HTML_ELEMENT_TYPE_TT:
 	case DOM_HTML_ELEMENT_TYPE_BDI:
 	case DOM_HTML_ELEMENT_TYPE_BDO:
+	case DOM_HTML_ELEMENT_TYPE_BIG:
 	case DOM_HTML_ELEMENT_TYPE_DFN:
 	case DOM_HTML_ELEMENT_TYPE_KBD:
 	case DOM_HTML_ELEMENT_TYPE_NAV:
@@ -819,6 +842,7 @@ _dom_html_document_create_element_internal(
 	case DOM_HTML_ELEMENT_TYPE_CODE:
 	case DOM_HTML_ELEMENT_TYPE_MAIN:
 	case DOM_HTML_ELEMENT_TYPE_MARK:
+	case DOM_HTML_ELEMENT_TYPE_NOBR:
 	case DOM_HTML_ELEMENT_TYPE_RUBY:
 	case DOM_HTML_ELEMENT_TYPE_SAMP:
 	case DOM_HTML_ELEMENT_TYPE_ASIDE:
@@ -829,10 +853,16 @@ _dom_html_document_create_element_internal(
 	case DOM_HTML_ELEMENT_TYPE_HEADER:
 	case DOM_HTML_ELEMENT_TYPE_HGROUP:
 	case DOM_HTML_ELEMENT_TYPE_STRONG:
+	case DOM_HTML_ELEMENT_TYPE_SPACER:
+	case DOM_HTML_ELEMENT_TYPE_STRIKE:
+	case DOM_HTML_ELEMENT_TYPE_ACRONYM:
 	case DOM_HTML_ELEMENT_TYPE_ADDRESS:
 	case DOM_HTML_ELEMENT_TYPE_ARTICLE:
+	case DOM_HTML_ELEMENT_TYPE_BGSOUND:
+	case DOM_HTML_ELEMENT_TYPE_MARQUEE:
 	case DOM_HTML_ELEMENT_TYPE_SECTION:
 	case DOM_HTML_ELEMENT_TYPE_SUMMARY:
+	case DOM_HTML_ELEMENT_TYPE_NOFRAMES:
 	case DOM_HTML_ELEMENT_TYPE_NOSCRIPT:
 	case DOM_HTML_ELEMENT_TYPE_FIGCAPTION:
 		/* These have no specialisation: use HTMLElement */
