@@ -16,7 +16,7 @@
 #include "core/attr.h"
 #include "utils/utils.h"
 
-static struct dom_element_protected_vtable _protect_vtable = {
+static const struct dom_element_protected_vtable _protect_vtable = {
 	{
 		DOM_NODE_PROTECT_VTABLE_HTML_ISINDEX_ELEMENT
 	},
