@@ -228,7 +228,10 @@ int main(int argc, char **argv)
 			buf_clear(&got);
 			buf_clear(&expected);
 
-			hubbub_parser_destroy(parser);
+			if (parser != NULL) {
+				hubbub_parser_destroy(parser);
+				parser = NULL;
+			}
 			while (Document) {
 				node_t *victim = Document;
 				Document = victim->next;
@@ -237,6 +240,7 @@ int main(int argc, char **argv)
 			Document = NULL;
 
 			state = EXPECT_DATA;
+			/* Fall through. */
 
  		case EXPECT_DATA:
 			if (strcmp(line, "#data\n") == 0) {
@@ -315,7 +319,10 @@ int main(int argc, char **argv)
 			printf("%s", got.buf);
 		}
 
-		hubbub_parser_destroy(parser);
+		if (parser != NULL) {
+			hubbub_parser_destroy(parser);
+			parser = NULL;
+		}
 		while (Document) {
 			node_t *victim = Document;
 			Document = victim->next;
@@ -330,6 +337,10 @@ int main(int argc, char **argv)
 	free(got.buf);
 	free(expected.buf);
 
+	if (parser != NULL) {
+		hubbub_parser_destroy(parser);
+	}
+
 	return 0;
 }
 
@@ -878,9 +889,12 @@ static void node_print(buf_t *buf, node_t *node, unsigned depth)
 		buf_add(buf, node->data.element.name);
 		buf_add(buf, ">\n");
 
-		qsort(node->data.element.attrs, node->data.element.n_attrs,
-				sizeof *node->data.element.attrs,
-				compare_attrs);
+		if (node->data.element.n_attrs > 0) {
+			qsort(node->data.element.attrs,
+					node->data.element.n_attrs,
+					sizeof *node->data.element.attrs,
+					compare_attrs);
+		}
 
 		for (i = 0; i < node->data.element.n_attrs; i++) {
 			indent(buf, depth + 1);
