@@ -52,7 +52,7 @@ with_simple_context_setup(void)
 	memset(captured_rendered_message, 0, sizeof(captured_rendered_message));
 	captured_rendered_message_length = 0;
 	captured_message_count = 0;
-	fail_unless(nslog_set_render_callback(
+	ck_assert_msg(nslog_set_render_callback(
 			    nslog__test__render_function,
 			    (void *)anchor_context_1) == NSLOG_NO_ERROR,
 		    "Unable to set up render callback");
@@ -67,65 +67,65 @@ with_simple_context_teardown(void)
 START_TEST (test_nslog_trivial_corked_message)
 {
 	NSLOG(test, INFO, "Hello %s", "world");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong");
-	fail_unless(captured_render_context == anchor_context_1,
+	ck_assert_msg(captured_render_context == anchor_context_1,
 		    "Captured context wasn't passed through");
-	fail_unless(strcmp(captured_context.category->name, "test") == 0,
+	ck_assert_msg(strcmp(captured_context.category->name, "test") == 0,
 		    "Captured context category wasn't normalised");
-	fail_unless(captured_context.category == &__nslog_category_test,
+	ck_assert_msg(captured_context.category == &__nslog_category_test,
 		    "Captured context category wasn't the one we wanted");
-	fail_unless(captured_rendered_message_length == 11,
+	ck_assert_msg(captured_rendered_message_length == 11,
 		    "Captured message wasn't correct length");
-	fail_unless(strcmp(captured_rendered_message, "Hello world") == 0,
+	ck_assert_msg(strcmp(captured_rendered_message, "Hello world") == 0,
 		    "Captured message wasn't correct");
-	fail_unless(strcmp(captured_context.filename, "test/basictests.c") == 0,
+	ck_assert_msg(strcmp(captured_context.filename, "test/basictests.c") == 0,
 		    "Captured message wasn't correct filename");
-	fail_unless(strcmp(captured_context.funcname, "test_nslog_trivial_corked_message") == 0,
+	ck_assert_msg(strcmp(captured_context.funcname, __func__) == 0,
 		    "Captured message wasn't correct function name");
 }
 END_TEST
 
 START_TEST (test_nslog_trivial_uncorked_message)
 {
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
-	fail_unless(captured_message_count == 0,
+	ck_assert_msg(captured_message_count == 0,
 		    "Unusual, we had messages from before uncorking");
 	NSLOG(test, INFO, "Hello %s", "world");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong");
-	fail_unless(captured_render_context == anchor_context_1,
+	ck_assert_msg(captured_render_context == anchor_context_1,
 		    "Captured context wasn't passed through");
-	fail_unless(strcmp(captured_context.category->name, "test") == 0,
+	ck_assert_msg(strcmp(captured_context.category->name, "test") == 0,
 		    "Captured context category wasn't normalised");
-	fail_unless(captured_context.category == &__nslog_category_test,
+	ck_assert_msg(captured_context.category == &__nslog_category_test,
 		    "Captured context category wasn't the one we wanted");
-	fail_unless(captured_rendered_message_length == 11,
+	ck_assert_msg(captured_rendered_message_length == 11,
 		    "Captured message wasn't correct length");
-	fail_unless(strcmp(captured_rendered_message, "Hello world") == 0,
+	ck_assert_msg(strcmp(captured_rendered_message, "Hello world") == 0,
 		    "Captured message wasn't correct");
-	fail_unless(strcmp(captured_context.filename, "test/basictests.c") == 0,
+	ck_assert_msg(strcmp(captured_context.filename, "test/basictests.c") == 0,
 		    "Captured message wasn't correct filename");
-	fail_unless(strcmp(captured_context.funcname, "test_nslog_trivial_uncorked_message") == 0,
+	ck_assert_msg(strcmp(captured_context.funcname, __func__) == 0,
 		    "Captured message wasn't correct function name");
 }
 END_TEST
 
 START_TEST (test_nslog_subcategory_name)
 {
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
-	fail_unless(captured_message_count == 0,
+	ck_assert_msg(captured_message_count == 0,
 		    "Unusual, we had messages from before uncorking");
 	NSLOG(sub, INFO, "Hello %s", "world");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong");
-	fail_unless(captured_render_context == anchor_context_1,
+	ck_assert_msg(captured_render_context == anchor_context_1,
 		    "Captured context wasn't passed through");
-	fail_unless(strcmp(captured_context.category->name, "test/sub") == 0,
+	ck_assert_msg(strcmp(captured_context.category->name, "test/sub") == 0,
 		    "Captured context category wasn't normalised");
 }
 END_TEST
@@ -134,30 +134,30 @@ START_TEST (test_nslog_two_corked_messages)
 {
 	NSLOG(test, INFO, "First");
 	NSLOG(sub, CRIT, "Second");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
-	fail_unless(captured_message_count == 2,
+	ck_assert_msg(captured_message_count == 2,
 		    "Captured message count was wrong");
-	fail_unless(captured_render_context == anchor_context_1,
+	ck_assert_msg(captured_render_context == anchor_context_1,
 		    "Captured context wasn't passed through");
-	fail_unless(strcmp(captured_context.category->name, "test/sub") == 0,
+	ck_assert_msg(strcmp(captured_context.category->name, "test/sub") == 0,
 		    "Captured context category wasn't normalised");
-	fail_unless(captured_context.category == &__nslog_category_sub,
+	ck_assert_msg(captured_context.category == &__nslog_category_sub,
 		    "Captured context category wasn't the one we wanted");
-	fail_unless(captured_rendered_message_length == 6,
+	ck_assert_msg(captured_rendered_message_length == 6,
 		    "Captured message wasn't correct length");
-	fail_unless(strcmp(captured_rendered_message, "Second") == 0,
+	ck_assert_msg(strcmp(captured_rendered_message, "Second") == 0,
 		    "Captured message wasn't correct");
-	fail_unless(strcmp(captured_context.filename, "test/basictests.c") == 0,
+	ck_assert_msg(strcmp(captured_context.filename, "test/basictests.c") == 0,
 		    "Captured message wasn't correct filename");
-	fail_unless(strcmp(captured_context.funcname, "test_nslog_two_corked_messages") == 0,
+	ck_assert_msg(strcmp(captured_context.funcname, __func__) == 0,
 		    "Captured message wasn't correct function name");
 }
 END_TEST
 
 START_TEST (test_nslog_check_bad_level)
 {
-	fail_unless(strcmp(nslog_level_name((nslog_level)-1),
+	ck_assert_msg(strcmp(nslog_level_name((nslog_level)-1),
 			   "**UNKNOWN**") == 0,
 		    "Failed to fail to render bad level");
 }
@@ -179,22 +179,22 @@ with_simple_filter_context_setup(void)
 	memset(captured_rendered_message, 0, sizeof(captured_rendered_message));
 	captured_rendered_message_length = 0;
 	captured_message_count = 0;
-	fail_unless(nslog_set_render_callback(
+	ck_assert_msg(nslog_set_render_callback(
 			    nslog__test__render_function,
 			    (void *)anchor_context_2) == NSLOG_NO_ERROR,
 		    "Unable to set up render callback");
-	fail_unless(nslog_filter_category_new("test", &cat_test) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_category_new("test", &cat_test) == NSLOG_NO_ERROR,
 		    "Unable to create a category filter for 'test'");
-	fail_unless(nslog_filter_category_new("another", &cat_another) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_category_new("another", &cat_another) == NSLOG_NO_ERROR,
 		    "Unable to create a category filter for 'another'");
-	fail_unless(nslog_filter_category_new("test/sub", &cat_test_sub) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_category_new("test/sub", &cat_test_sub) == NSLOG_NO_ERROR,
 		    "Unable to create a category filter for 'test/sub'");
 }
 
 static void
 with_simple_filter_context_teardown(void)
 {
-	fail_unless(nslog_filter_set_active(NULL, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(NULL, NULL) == NSLOG_NO_ERROR,
 		    "Unable to clear active filter");
 	cat_test = nslog_filter_unref(cat_test);
 	cat_another = nslog_filter_unref(cat_another);
@@ -205,25 +205,25 @@ with_simple_filter_context_teardown(void)
 START_TEST (test_nslog_simple_filter_corked_message)
 {
 	NSLOG(test, INFO, "Hello world");
-	fail_unless(nslog_filter_set_active(cat_test, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(cat_test, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to cat:test");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong");
-	fail_unless(captured_render_context == anchor_context_2,
+	ck_assert_msg(captured_render_context == anchor_context_2,
 		    "Captured context wasn't passed through");
-	fail_unless(strcmp(captured_context.category->name, "test") == 0,
+	ck_assert_msg(strcmp(captured_context.category->name, "test") == 0,
 		    "Captured context category wasn't normalised");
-	fail_unless(captured_context.category == &__nslog_category_test,
+	ck_assert_msg(captured_context.category == &__nslog_category_test,
 		    "Captured context category wasn't the one we wanted");
-	fail_unless(captured_rendered_message_length == 11,
+	ck_assert_msg(captured_rendered_message_length == 11,
 		    "Captured message wasn't correct length");
-	fail_unless(strcmp(captured_rendered_message, "Hello world") == 0,
+	ck_assert_msg(strcmp(captured_rendered_message, "Hello world") == 0,
 		    "Captured message wasn't correct");
-	fail_unless(strcmp(captured_context.filename, "test/basictests.c") == 0,
+	ck_assert_msg(strcmp(captured_context.filename, "test/basictests.c") == 0,
 		    "Captured message wasn't correct filename");
-	fail_unless(strcmp(captured_context.funcname, "test_nslog_simple_filter_corked_message") == 0,
+	ck_assert_msg(strcmp(captured_context.funcname, __func__) == 0,
 		    "Captured message wasn't correct function name");
 
 }
@@ -231,26 +231,26 @@ END_TEST
 
 START_TEST (test_nslog_simple_filter_uncorked_message)
 {
-	fail_unless(nslog_filter_set_active(cat_test, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(cat_test, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to cat:test");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
 	NSLOG(test, INFO, "Hello world");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong");
-	fail_unless(captured_render_context == anchor_context_2,
+	ck_assert_msg(captured_render_context == anchor_context_2,
 		    "Captured context wasn't passed through");
-	fail_unless(strcmp(captured_context.category->name, "test") == 0,
+	ck_assert_msg(strcmp(captured_context.category->name, "test") == 0,
 		    "Captured context category wasn't normalised");
-	fail_unless(captured_context.category == &__nslog_category_test,
+	ck_assert_msg(captured_context.category == &__nslog_category_test,
 		    "Captured context category wasn't the one we wanted");
-	fail_unless(captured_rendered_message_length == 11,
+	ck_assert_msg(captured_rendered_message_length == 11,
 		    "Captured message wasn't correct length");
-	fail_unless(strcmp(captured_rendered_message, "Hello world") == 0,
+	ck_assert_msg(strcmp(captured_rendered_message, "Hello world") == 0,
 		    "Captured message wasn't correct");
-	fail_unless(strcmp(captured_context.filename, "test/basictests.c") == 0,
+	ck_assert_msg(strcmp(captured_context.filename, "test/basictests.c") == 0,
 		    "Captured message wasn't correct filename");
-	fail_unless(strcmp(captured_context.funcname, "test_nslog_simple_filter_uncorked_message") == 0,
+	ck_assert_msg(strcmp(captured_context.funcname, __func__) == 0,
 		    "Captured message wasn't correct function name");
 
 }
@@ -258,26 +258,26 @@ END_TEST
 
 START_TEST (test_nslog_simple_filter_subcategory_message)
 {
-	fail_unless(nslog_filter_set_active(cat_test, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(cat_test, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to cat:test");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
 	NSLOG(sub, INFO, "Hello world");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong");
-	fail_unless(captured_render_context == anchor_context_2,
+	ck_assert_msg(captured_render_context == anchor_context_2,
 		    "Captured context wasn't passed through");
-	fail_unless(strcmp(captured_context.category->name, "test/sub") == 0,
+	ck_assert_msg(strcmp(captured_context.category->name, "test/sub") == 0,
 		    "Captured context category wasn't normalised");
-	fail_unless(captured_context.category == &__nslog_category_sub,
+	ck_assert_msg(captured_context.category == &__nslog_category_sub,
 		    "Captured context category wasn't the one we wanted");
-	fail_unless(captured_rendered_message_length == 11,
+	ck_assert_msg(captured_rendered_message_length == 11,
 		    "Captured message wasn't correct length");
-	fail_unless(strcmp(captured_rendered_message, "Hello world") == 0,
+	ck_assert_msg(strcmp(captured_rendered_message, "Hello world") == 0,
 		    "Captured message wasn't correct");
-	fail_unless(strcmp(captured_context.filename, "test/basictests.c") == 0,
+	ck_assert_msg(strcmp(captured_context.filename, "test/basictests.c") == 0,
 		    "Captured message wasn't correct filename");
-	fail_unless(strcmp(captured_context.funcname, "test_nslog_simple_filter_subcategory_message") == 0,
+	ck_assert_msg(strcmp(captured_context.funcname, __func__) == 0,
 		    "Captured message wasn't correct function name");
 
 }
@@ -285,12 +285,12 @@ END_TEST
 
 START_TEST (test_nslog_simple_filter_out_subcategory_message)
 {
-	fail_unless(nslog_filter_set_active(cat_test_sub, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(cat_test_sub, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to cat:test/sub");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
 	NSLOG(test, INFO, "Hello world");
-	fail_unless(captured_message_count == 0,
+	ck_assert_msg(captured_message_count == 0,
 		    "Captured message count was wrong");
 }
 END_TEST
@@ -298,13 +298,13 @@ END_TEST
 START_TEST (test_nslog_basic_filter_sprintf)
 {
 	char *ct = nslog_filter_sprintf(cat_test);
-	fail_unless(ct != NULL, "Unable to sprintf");
-	fail_unless(strcmp(ct, "cat:test") == 0,
+	ck_assert_msg(ct != NULL, "Unable to sprintf");
+	ck_assert_msg(strcmp(ct, "cat:test") == 0,
 		    "Printed category test is wrong");
 	free(ct);
 	ct = nslog_filter_sprintf(cat_another);
-	fail_unless(ct != NULL, "Unable to sprintf");
-	fail_unless(strcmp(ct, "cat:another") == 0,
+	ck_assert_msg(ct != NULL, "Unable to sprintf");
+	ck_assert_msg(strcmp(ct, "cat:another") == 0,
 		    "Printed category another is wrong");
 	free(ct);
 }
@@ -313,13 +313,13 @@ END_TEST
 START_TEST (test_nslog_parse_and_sprintf)
 {
 	nslog_filter_t *filt = NULL;
-	fail_unless(nslog_filter_from_text("cat:test", &filt) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_from_text("cat:test", &filt) == NSLOG_NO_ERROR,
 		    "Unable to parse cat:test");
-	fail_unless(filt != NULL,
+	ck_assert_msg(filt != NULL,
 		    "Strange, despite parsing okay, filt was NULL");
 	char *ct = nslog_filter_sprintf(filt);
 	nslog_filter_unref(filt);
-	fail_unless(strcmp(ct, "cat:test") == 0,
+	ck_assert_msg(strcmp(ct, "cat:test") == 0,
 		    "Printed parsed cat:test not right");
 	free(ct);
 }
@@ -330,13 +330,13 @@ START_TEST (test_nslog_parse_and_sprintf_all_levels)
 	nslog_filter_t *filt = NULL;
 	const char *input =
 		"((((((lvl:DEEPDEBUG || lvl:DEBUG) || lvl:VERBOSE) || lvl:INFO) || lvl:WARNING) || lvl:ERROR) || lvl:CRITICAL)";
-	fail_unless(nslog_filter_from_text(input, &filt) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_from_text(input, &filt) == NSLOG_NO_ERROR,
 		    "Unable to parse all level test");
-	fail_unless(filt != NULL,
+	ck_assert_msg(filt != NULL,
 		    "Strange, despite parsing okay, filt was NULL");
 	char *ct = nslog_filter_sprintf(filt);
 	nslog_filter_unref(filt);
-	fail_unless(strcmp(ct, input) == 0,
+	ck_assert_msg(strcmp(ct, input) == 0,
 		    "Printed parsed all-level not right");
 	free(ct);
 }
@@ -347,13 +347,13 @@ START_TEST (test_nslog_parse_and_sprintf_all_kinds)
 	nslog_filter_t *filt = NULL;
 	const char *input =
 		"!((((lvl:WARNING || cat:test) && file:foo) ^ dir:bar) || func:baz)";
-	fail_unless(nslog_filter_from_text(input, &filt) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_from_text(input, &filt) == NSLOG_NO_ERROR,
 		    "Unable to parse all kind test");
-	fail_unless(filt != NULL,
+	ck_assert_msg(filt != NULL,
 		    "Strange, despite parsing okay, filt was NULL");
 	char *ct = nslog_filter_sprintf(filt);
 	nslog_filter_unref(filt);
-	fail_unless(strcmp(ct, input) == 0,
+	ck_assert_msg(strcmp(ct, input) == 0,
 		    "Printed parsed all-kind not right");
 	free(ct);
 }
@@ -371,7 +371,7 @@ with_trivial_filter_context_setup(void)
 	memset(captured_rendered_message, 0, sizeof(captured_rendered_message));
 	captured_rendered_message_length = 0;
 	captured_message_count = 0;
-	fail_unless(nslog_set_render_callback(
+	ck_assert_msg(nslog_set_render_callback(
 			    nslog__test__render_function,
 			    (void *)anchor_context_3) == NSLOG_NO_ERROR,
 		    "Unable to set up render callback");
@@ -380,7 +380,7 @@ with_trivial_filter_context_setup(void)
 static void
 with_trivial_filter_context_teardown(void)
 {
-	fail_unless(nslog_filter_set_active(NULL, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(NULL, NULL) == NSLOG_NO_ERROR,
 		    "Unable to clear active filter");
 	nslog_cleanup();
 }
@@ -388,20 +388,20 @@ with_trivial_filter_context_teardown(void)
 START_TEST (test_nslog_filter_filename)
 {
 	nslog_filter_t *filter;
-	fail_unless(nslog_filter_filename_new("basictests.c", &filter) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_filename_new("basictests.c", &filter) == NSLOG_NO_ERROR,
 		    "Unable to create filename filter");
-	fail_unless(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to file:basictests.c");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
 	NSLOG(test, WARN, "Hello");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong");
-	fail_unless(captured_render_context == anchor_context_3,
+	ck_assert_msg(captured_render_context == anchor_context_3,
 		    "Captured context wasn't passed through");
-	fail_unless(captured_rendered_message_length == 5,
+	ck_assert_msg(captured_rendered_message_length == 5,
 		    "Captured message wasn't correct length");
-	fail_unless(strcmp(captured_rendered_message, "Hello") == 0,
+	ck_assert_msg(strcmp(captured_rendered_message, "Hello") == 0,
 		    "Mesage wasn't as expected");
 	filter = nslog_filter_unref(filter);
 }
@@ -410,20 +410,20 @@ END_TEST
 START_TEST (test_nslog_filter_full_filename)
 {
 	nslog_filter_t *filter;
-	fail_unless(nslog_filter_filename_new("test/basictests.c", &filter) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_filename_new("test/basictests.c", &filter) == NSLOG_NO_ERROR,
 		    "Unable to create filename filter");
-	fail_unless(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to file:test/basictests.c");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
 	NSLOG(test, WARN, "Hello");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong");
-	fail_unless(captured_render_context == anchor_context_3,
+	ck_assert_msg(captured_render_context == anchor_context_3,
 		    "Captured context wasn't passed through");
-	fail_unless(captured_rendered_message_length == 5,
+	ck_assert_msg(captured_rendered_message_length == 5,
 		    "Captured message wasn't correct length");
-	fail_unless(strcmp(captured_rendered_message, "Hello") == 0,
+	ck_assert_msg(strcmp(captured_rendered_message, "Hello") == 0,
 		    "Mesage wasn't as expected");
 	filter = nslog_filter_unref(filter);
 }
@@ -432,14 +432,14 @@ END_TEST
 START_TEST (test_nslog_filter_out_filename)
 {
 	nslog_filter_t *filter;
-	fail_unless(nslog_filter_filename_new("testmain.c", &filter) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_filename_new("testmain.c", &filter) == NSLOG_NO_ERROR,
 		    "Unable to create filename filter");
-	fail_unless(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to file:testmain.c");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
 	NSLOG(test, WARN, "Hello");
-	fail_unless(captured_message_count == 0,
+	ck_assert_msg(captured_message_count == 0,
 		    "Captured message count was wrong");
 	filter = nslog_filter_unref(filter);
 }
@@ -448,20 +448,20 @@ END_TEST
 START_TEST (test_nslog_filter_level)
 {
 	nslog_filter_t *filter;
-	fail_unless(nslog_filter_level_new(NSLOG_LEVEL_WARN, &filter) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_level_new(NSLOG_LEVEL_WARN, &filter) == NSLOG_NO_ERROR,
 		    "Unable to create level filter");
-	fail_unless(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to lvl:WARN");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
 	NSLOG(test, WARN, "Hello");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong");
-	fail_unless(captured_render_context == anchor_context_3,
+	ck_assert_msg(captured_render_context == anchor_context_3,
 		    "Captured context wasn't passed through");
-	fail_unless(captured_rendered_message_length == 5,
+	ck_assert_msg(captured_rendered_message_length == 5,
 		    "Captured message wasn't correct length");
-	fail_unless(strcmp(captured_rendered_message, "Hello") == 0,
+	ck_assert_msg(strcmp(captured_rendered_message, "Hello") == 0,
 		    "Mesage wasn't as expected");
 	filter = nslog_filter_unref(filter);
 }
@@ -470,14 +470,14 @@ END_TEST
 START_TEST (test_nslog_filter_out_level)
 {
 	nslog_filter_t *filter;
-	fail_unless(nslog_filter_level_new(NSLOG_LEVEL_ERR, &filter) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_level_new(NSLOG_LEVEL_ERR, &filter) == NSLOG_NO_ERROR,
 		    "Unable to create filename filter");
-	fail_unless(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to lvl:ERR");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
 	NSLOG(test, WARN, "Hello");
-	fail_unless(captured_message_count == 0,
+	ck_assert_msg(captured_message_count == 0,
 		    "Captured message count was wrong");
 	filter = nslog_filter_unref(filter);
 }
@@ -486,20 +486,20 @@ END_TEST
 START_TEST (test_nslog_filter_dirname)
 {
 	nslog_filter_t *filter;
-	fail_unless(nslog_filter_dirname_new("test", &filter) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_dirname_new("test", &filter) == NSLOG_NO_ERROR,
 		    "Unable to create level filter");
-	fail_unless(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to dir:test");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
 	NSLOG(test, WARN, "Hello");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong");
-	fail_unless(captured_render_context == anchor_context_3,
+	ck_assert_msg(captured_render_context == anchor_context_3,
 		    "Captured context wasn't passed through");
-	fail_unless(captured_rendered_message_length == 5,
+	ck_assert_msg(captured_rendered_message_length == 5,
 		    "Captured message wasn't correct length");
-	fail_unless(strcmp(captured_rendered_message, "Hello") == 0,
+	ck_assert_msg(strcmp(captured_rendered_message, "Hello") == 0,
 		    "Mesage wasn't as expected");
 	filter = nslog_filter_unref(filter);
 }
@@ -508,14 +508,14 @@ END_TEST
 START_TEST (test_nslog_filter_out_dirname)
 {
 	nslog_filter_t *filter;
-	fail_unless(nslog_filter_dirname_new("src", &filter) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_dirname_new("src", &filter) == NSLOG_NO_ERROR,
 		    "Unable to create filename filter");
-	fail_unless(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to dir:src");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
 	NSLOG(test, WARN, "Hello");
-	fail_unless(captured_message_count == 0,
+	ck_assert_msg(captured_message_count == 0,
 		    "Captured message count was wrong");
 	filter = nslog_filter_unref(filter);
 }
@@ -524,20 +524,20 @@ END_TEST
 START_TEST (test_nslog_filter_funcname)
 {
 	nslog_filter_t *filter;
-	fail_unless(nslog_filter_funcname_new("test_nslog_filter_funcname", &filter) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_funcname_new(__func__, &filter) == NSLOG_NO_ERROR,
 		    "Unable to create level filter");
-	fail_unless(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to dir:test");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
 	NSLOG(test, WARN, "Hello");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong");
-	fail_unless(captured_render_context == anchor_context_3,
+	ck_assert_msg(captured_render_context == anchor_context_3,
 		    "Captured context wasn't passed through");
-	fail_unless(captured_rendered_message_length == 5,
+	ck_assert_msg(captured_rendered_message_length == 5,
 		    "Captured message wasn't correct length");
-	fail_unless(strcmp(captured_rendered_message, "Hello") == 0,
+	ck_assert_msg(strcmp(captured_rendered_message, "Hello") == 0,
 		    "Mesage wasn't as expected");
 	filter = nslog_filter_unref(filter);
 }
@@ -546,14 +546,14 @@ END_TEST
 START_TEST (test_nslog_filter_out_funcname)
 {
 	nslog_filter_t *filter;
-	fail_unless(nslog_filter_funcname_new("test_nslog_filter_funcname", &filter) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_funcname_new("test_nslog_filter_funcname", &filter) == NSLOG_NO_ERROR,
 		    "Unable to create filename filter");
-	fail_unless(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to dir:src");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
 	NSLOG(test, WARN, "Hello");
-	fail_unless(captured_message_count == 0,
+	ck_assert_msg(captured_message_count == 0,
 		    "Captured message count was wrong");
 	filter = nslog_filter_unref(filter);
 }
@@ -562,20 +562,20 @@ END_TEST
 START_TEST (test_nslog_complex_filter1)
 {
 	nslog_filter_t *filter;
-	fail_unless(nslog_filter_from_text("(lvl:WARN || (lvl:DEBUG && cat:test/sub))", &filter) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_from_text("(lvl:WARN || (lvl:DEBUG && cat:test/sub))", &filter) == NSLOG_NO_ERROR,
 		    "Unable to create filename filter");
-	fail_unless(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to dir:src");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
 	NSLOG(sub, WARN, "Hello");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong (1)");
 	NSLOG(test, DEBUG, "Hello");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong (2)");
 	NSLOG(sub, DEBUG, "Hello");
-	fail_unless(captured_message_count == 2,
+	ck_assert_msg(captured_message_count == 2,
 		    "Captured message count was wrong (3)");
 	filter = nslog_filter_unref(filter);
 }
@@ -584,20 +584,20 @@ END_TEST
 START_TEST (test_nslog_complex_filter2)
 {
 	nslog_filter_t *filter;
-	fail_unless(nslog_filter_from_text("!(lvl:WARN ^ cat:test/sub)", &filter) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_from_text("!(lvl:WARN ^ cat:test/sub)", &filter) == NSLOG_NO_ERROR,
 		    "Unable to create filename filter");
-	fail_unless(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_filter_set_active(filter, NULL) == NSLOG_NO_ERROR,
 		    "Unable to set active filter to dir:src");
-	fail_unless(nslog_uncork() == NSLOG_NO_ERROR,
+	ck_assert_msg(nslog_uncork() == NSLOG_NO_ERROR,
 		    "Unable to uncork");
 	NSLOG(sub, WARN, "Hello");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong (1)");
 	NSLOG(test, WARN, "Hello");
-	fail_unless(captured_message_count == 1,
+	ck_assert_msg(captured_message_count == 1,
 		    "Captured message count was wrong (2)");
 	NSLOG(test, DEBUG, "Hello");
-	fail_unless(captured_message_count == 2,
+	ck_assert_msg(captured_message_count == 2,
 		    "Captured message count was wrong (3)");
 	filter = nslog_filter_unref(filter);
 }
