@@ -19,6 +19,8 @@
 #include <stdio.h>
 #include <libcss/libcss.h>
 
+#include "netsurf/inttypes.h"
+
 #include "css/dump.h"
 #include "css/utils.h"
 
@@ -36,7 +38,7 @@ static void dump_css_fixed(FILE *stream, css_fixed f)
 	uint32_t fracpart = ((NSCSS_ABS(f) & 0x3ff) * 1000 + 500) / (1 << 10);
 #undef NSCSS_ABS
 
-	fprintf(stream, "%s%d.%03d", f < 0 ? "-" : "", uintpart, fracpart);
+	fprintf(stream, "%s%"PRIu32".%03"PRIu32, f < 0 ? "-" : "", uintpart, fracpart);
 }
 
 /**
@@ -48,7 +50,7 @@ static void dump_css_fixed(FILE *stream, css_fixed f)
 static void dump_css_number(FILE *stream, css_fixed val)
 {
 	if (INTTOFIX(FIXTOINT(val)) == val)
-		fprintf(stream, "%d", FIXTOINT(val));
+		fprintf(stream, "%"PRId32, FIXTOINT(val));
 	else
 		dump_css_fixed(stream, val);
 }
@@ -113,24 +115,15 @@ static void dump_css_unit(FILE *stream, css_fixed val, css_unit unit)
 	case CSS_UNIT_KHZ:
 		fprintf(stream, "kHz");
 		break;
-	case CSS_UNIT_CAP:
-		fprintf(stream, "cap");
-		break;
 	case CSS_UNIT_CH:
 		fprintf(stream, "ch");
 		break;
-	case CSS_UNIT_IC:
-		fprintf(stream, "ic");
-		break;
 	case CSS_UNIT_REM:
 		fprintf(stream, "rem");
 		break;
 	case CSS_UNIT_LH:
 		fprintf(stream, "lh");
 		break;
-	case CSS_UNIT_RLH:
-		fprintf(stream, "rlh");
-		break;
 	case CSS_UNIT_VH:
 		fprintf(stream, "vh");
 		break;
@@ -190,7 +183,7 @@ void nscss_dump_computed_style(FILE *stream, const css_computed_style *style)
 	val = css_computed_background_color(style, &color);
 	switch (val) {
 	case CSS_BACKGROUND_COLOR_COLOR:
-		fprintf(stream, "background-color: #%08x ", color);
+		fprintf(stream, "background-color: #%08"PRIx32" ", color);
 		break;
 	default:
 		break;
@@ -264,7 +257,7 @@ void nscss_dump_computed_style(FILE *stream, const css_computed_style *style)
 	val = css_computed_border_top_color(style, &color);
 	switch (val) {
 	case CSS_BORDER_COLOR_COLOR:
-		fprintf(stream, "border-top-color: #%08x ", color);
+		fprintf(stream, "border-top-color: #%08"PRIx32" ", color);
 		break;
 	default:
 		break;
@@ -274,7 +267,7 @@ void nscss_dump_computed_style(FILE *stream, const css_computed_style *style)
 	val = css_computed_border_right_color(style, &color);
 	switch (val) {
 	case CSS_BORDER_COLOR_COLOR:
-		fprintf(stream, "border-right-color: #%08x ", color);
+		fprintf(stream, "border-right-color: #%08"PRIx32" ", color);
 		break;
 	default:
 		break;
@@ -284,7 +277,7 @@ void nscss_dump_computed_style(FILE *stream, const css_computed_style *style)
 	val = css_computed_border_bottom_color(style, &color);
 	switch (val) {
 	case CSS_BORDER_COLOR_COLOR:
-		fprintf(stream, "border-bottom-color: #%08x ", color);
+		fprintf(stream, "border-bottom-color: #%08"PRIx32" ", color);
 		break;
 	default:
 		break;
@@ -294,7 +287,7 @@ void nscss_dump_computed_style(FILE *stream, const css_computed_style *style)
 	val = css_computed_border_left_color(style, &color);
 	switch (val) {
 	case CSS_BORDER_COLOR_COLOR:
-		fprintf(stream, "border-left-color: #%08x ", color);
+		fprintf(stream, "border-left-color: #%08"PRIx32" ", color);
 		break;
 	default:
 		break;
@@ -619,7 +612,7 @@ void nscss_dump_computed_style(FILE *stream, const css_computed_style *style)
 	/* color */
 	val = css_computed_color(style, &color);
 	if (val == CSS_COLOR_COLOR) {
-		fprintf(stream, "color: #%08x ", color);
+		fprintf(stream, "color: #%08"PRIx32" ", color);
 	}
 
 	/* content */
@@ -873,6 +866,12 @@ void nscss_dump_computed_style(FILE *stream, const css_computed_style *style)
 	case CSS_DISPLAY_NONE:
 		fprintf(stream, "display: none ");
 		break;
+	case CSS_DISPLAY_FLEX:
+		fprintf(stream, "display: flex ");
+		break;
+	case CSS_DISPLAY_INLINE_FLEX:
+		fprintf(stream, "display: inline-flex ");
+		break;
 	default:
 		break;
 	}
@@ -1356,7 +1355,7 @@ void nscss_dump_computed_style(FILE *stream, const css_computed_style *style)
 		fprintf(stream, "outline-color: invert ");
 		break;
 	case CSS_OUTLINE_COLOR_COLOR:
-		fprintf(stream, "outline-color: #%08x ", color);
+		fprintf(stream, "outline-color: #%08"PRIx32" ", color);
 		break;
 	default:
 		break;
@@ -1823,7 +1822,7 @@ void nscss_dump_computed_style(FILE *stream, const css_computed_style *style)
 		fprintf(stream, "z-index: auto ");
 		break;
 	case CSS_Z_INDEX_SET:
-		fprintf(stream, "z-index: %d ", zindex);
+		fprintf(stream, "z-index: %"PRId32" ", zindex);
 		break;
 	default:
 		break;
