@@ -1579,14 +1579,64 @@ static void css_hint_white_space_nowrap(
 	}
 }
 
+static void css_hint_list(
+		nscss_select_ctx *ctx,
+		dom_node *node)
+{
+	struct css_hint *hint = &(hint_ctx.hints[hint_ctx.len]);
+	dom_exception err;
+	dom_string *attr;
 
-/* Exported function, documeted in css/hints.h */
+	err = dom_element_get_attribute(node, corestring_dom_type, &attr);
+	if (err == DOM_NO_ERR && attr != NULL) {
+		const char *attr_str = dom_string_data(attr);
+		size_t attr_len = dom_string_byte_length(attr);
+		enum css_list_style_type_e type = CSS_LIST_STYLE_TYPE_INHERIT;
+
+		if (attr_len == 1) {
+			switch (attr_str[0]) {
+			case 'a':
+				type = CSS_LIST_STYLE_TYPE_LOWER_ALPHA;
+				break;
+			case 'A':
+				type = CSS_LIST_STYLE_TYPE_UPPER_ALPHA;
+				break;
+			case 'i':
+				type = CSS_LIST_STYLE_TYPE_LOWER_ROMAN;
+				break;
+			case 'I':
+				type = CSS_LIST_STYLE_TYPE_UPPER_ROMAN;
+				break;
+			case '1':
+				type = CSS_LIST_STYLE_TYPE_DECIMAL;
+				break;
+			}
+		}
+
+		if (type != CSS_LIST_STYLE_TYPE_INHERIT) {
+			hint->prop = CSS_PROP_LIST_STYLE_TYPE;
+			hint->status = type;
+			css_hint_advance(&hint);
+		}
+
+		dom_string_unref(attr);
+	}
+}
+
+
+/* Exported function, documented in css/hints.h */
 css_error node_presentational_hint(void *pw, void *node,
 		uint32_t *nhints, css_hint **hints)
 {
 	dom_exception exc;
 	dom_html_element_type tag_type;
 
+	if (nsoption_bool(author_level_css) == false) {
+		*nhints = 0;
+		*hints = NULL;
+		return CSS_OK;
+	}
+
 	css_hint_clean();
 
 	exc = dom_html_element_get_tag_type(node, &tag_type);
@@ -1600,22 +1650,22 @@ css_error node_presentational_hint(void *pw, void *node,
 		css_hint_width(pw, node);
 		css_hint_table_cell_border_padding(pw, node);
 		css_hint_white_space_nowrap(pw, node);
-		/* fall through */
+		fallthrough;
 	case DOM_HTML_ELEMENT_TYPE_TR:
 		css_hint_height(pw, node);
-		/* fall through */
+		fallthrough;
 	case DOM_HTML_ELEMENT_TYPE_THEAD:
 	case DOM_HTML_ELEMENT_TYPE_TBODY:
 	case DOM_HTML_ELEMENT_TYPE_TFOOT:
 		css_hint_text_align_special(pw, node);
-		/* fall through */
+		fallthrough;
 	case DOM_HTML_ELEMENT_TYPE_COL:
 		css_hint_vertical_align_table_cells(pw, node);
 		break;
 	case DOM_HTML_ELEMENT_TYPE_APPLET:
 	case DOM_HTML_ELEMENT_TYPE_IMG:
 		css_hint_margin_hspace_vspace(pw, node);
-		/* fall through */
+		fallthrough;
 	case DOM_HTML_ELEMENT_TYPE_EMBED:
 	case DOM_HTML_ELEMENT_TYPE_IFRAME:
 	case DOM_HTML_ELEMENT_TYPE_OBJECT:
@@ -1638,7 +1688,7 @@ css_error node_presentational_hint(void *pw, void *node,
 		break;
 	case DOM_HTML_ELEMENT_TYPE_CAPTION:
 		css_hint_caption_side(pw, node);
-		/* fall through */
+		fallthrough;
 	case DOM_HTML_ELEMENT_TYPE_DIV:
 		css_hint_text_align_special(pw, node);
 		break;
@@ -1671,6 +1721,9 @@ css_error node_presentational_hint(void *pw, void *node,
 	case DOM_HTML_ELEMENT_TYPE_CANVAS:
 		css_hint_height_width_canvas(pw, node);
 		break;
+	case DOM_HTML_ELEMENT_TYPE_OL:
+		css_hint_list(pw, node);
+		break;
 	default:
 		break;
 	}
