@@ -28,6 +28,7 @@
 #undef DUK_USE_DATE_PARSE_STRING
 #endif
 
+#define DUK_USE_FASTINT
 #define DUK_USE_REGEXP_CANON_WORKAROUND
 
 /* Required for execution timeout checking */
@@ -35,3 +36,5 @@
 
 extern duk_bool_t dukky_check_timeout(void *udata);
 #define DUK_USE_EXEC_TIMEOUT_CHECK dukky_check_timeout
+
+#include "netsurf/inttypes.h"
