@@ -30,6 +30,7 @@
 #include "utils/messages.h"
 #include "utils/ring.h"
 #include "utils/utils.h"
+#include "netsurf/inttypes.h"
 #include "netsurf/misc.h"
 #include "netsurf/content.h"
 #include "desktop/gui_internal.h"
@@ -592,7 +593,7 @@ void hlcache_finalise(void)
 		num_contents++;
 	}
 
-	NSLOG(netsurf, INFO, "%d contents remain before cache drain",
+	NSLOG(netsurf, INFO, "%"PRIu32" contents remain before cache drain",
 	      num_contents);
 
 	/* Drain cache */
@@ -607,7 +608,8 @@ void hlcache_finalise(void)
 		}
 	} while (num_contents > 0 && num_contents != prev_contents);
 
-	NSLOG(netsurf, INFO, "%d contents remaining after being polite", num_contents);
+	NSLOG(netsurf, INFO, "%"PRIu32" contents remaining after being polite",
+			num_contents);
 
 	/* Drain cache again, forcing the matter */
 	do {
@@ -621,12 +623,12 @@ void hlcache_finalise(void)
 		}
 	} while (num_contents > 0 && num_contents != prev_contents);
 
-	NSLOG(netsurf, INFO, "%d contents remaining:", num_contents);
+	NSLOG(netsurf, INFO, "%"PRIu32" contents remaining:", num_contents);
 	for (entry = hlcache->content_list; entry != NULL; entry = entry->next) {
 		hlcache_handle entry_handle = { entry, NULL, NULL };
 
 		if (entry->content != NULL) {
-			NSLOG(netsurf, INFO, "	%p : %s (%d users)",
+			NSLOG(netsurf, INFO, "	%p : %s (%"PRIu32" users)",
 			      entry,
 			      nsurl_access(hlcache_handle_get_url(&entry_handle)),
 			      content_count_users(entry->content));
@@ -673,11 +675,15 @@ void hlcache_finalise(void)
 }
 
 /* See hlcache.h for documentation */
-nserror hlcache_handle_retrieve(nsurl *url, uint32_t flags,
-		nsurl *referer, llcache_post_data *post,
-		hlcache_handle_callback cb, void *pw,
-		hlcache_child_context *child,
-		content_type accepted_types, hlcache_handle **result)
+nserror
+hlcache_handle_retrieve(nsurl *url,
+			uint32_t flags,
+			nsurl *referer,
+			llcache_post_data *post,
+			hlcache_handle_callback cb, void *pw,
+			hlcache_child_context *child,
+			content_type accepted_types,
+			hlcache_handle **result)
 {
 	hlcache_retrieval_ctx *ctx;
 	nserror error;
