@@ -16,8 +16,9 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
- * Browser window creation and manipulation (interface).
+/**
+ * \file
+ * Interface to browser frames.
  */
 
 #ifndef _NETSURF_DESKTOP_FRAME_TYPES_H_
@@ -32,12 +33,6 @@ struct frame_dimension {
 	} unit;
 };
 
-typedef enum {
-  	SCROLLING_AUTO,
-  	SCROLLING_YES,
-  	SCROLLING_NO
-} frame_scrolling;
-
 /* Handy struct names */
 struct content_html_iframe;
 struct content_html_frames;
