@@ -104,6 +104,9 @@ NSOPTION_INTEGER(disc_cache_age, 28)
 /** Whether to block advertisements */
 NSOPTION_BOOL(block_advertisements, false)
 
+/** Control popups e.g. noniteractive opening windows with window.open */
+NSOPTION_BOOL(disable_popups, false)
+
 /** Disable website tracking, see	                
  * http://www.w3.org/Submission/2011/SUBM-web-tracking-protection-20110224/#dnt-uas */
 NSOPTION_BOOL(do_not_track, false)
@@ -123,6 +126,9 @@ NSOPTION_BOOL(animate_images, true)
 /** Whether to execute javascript */
 NSOPTION_BOOL(enable_javascript, false)
 
+/** Whether to allow Author level CSS. */
+NSOPTION_BOOL(author_level_css, true)
+
 /** Maximum time (in seconds) to wait for a script to run */
 NSOPTION_INTEGER(script_timeout, 10)
 
@@ -151,7 +157,7 @@ NSOPTION_STRING(homepage_url, NULL)
 NSOPTION_BOOL(search_url_bar, false)
 
 /** default web search provider */
-NSOPTION_INTEGER(search_provider, 0)
+NSOPTION_STRING(search_web_provider, NULL)
 
 /** URL completion in url bar */
 NSOPTION_BOOL(url_suggestion, true)
@@ -217,12 +223,17 @@ NSOPTION_UINT(curl_fetch_timeout, 30)
 /** Suppress debug output from cURL. */
 NSOPTION_BOOL(suppress_curl_debug, true)
 
+/******** appearnce of new browser views ********/
+
 /** Whether to allow target="_blank" */
 NSOPTION_BOOL(target_blank, true)
 
-/** Whether second mouse button opens in new tab */
+/** auxillary (second) mouse button opens in new tab */
 NSOPTION_BOOL(button_2_tab, true)
 
+/** control if new view (window or tab) opens in foreground */
+NSOPTION_BOOL(foreground_new, false)
+
 /******** PDF / Print options ********/
 
 /** top margin of exported page */
@@ -255,6 +266,9 @@ NSOPTION_BOOL(enable_PDF_compression, true)
 /** setting a password and encoding PDF documents */
 NSOPTION_BOOL(enable_PDF_password, false)
 
+/** whether to prefer dark mode (light on dark) */
+NSOPTION_BOOL(prefer_dark_mode, false)
+
 /******** System colours ********/
 NSOPTION_COLOUR(sys_colour_ActiveBorder, 0x00d3d3d3)
 NSOPTION_COLOUR(sys_colour_ActiveCaption, 0x00f1f1f1)
