@@ -21,8 +21,8 @@ struct hlcache_object;
 struct selection;
 struct gui_window_2;
 
-struct FileRequester *filereq;
-struct FileRequester *savereq;
+extern struct FileRequester *filereq;
+extern struct FileRequester *savereq;
 
 enum {
 	AMINS_SAVE_SOURCE,
