@@ -20,8 +20,6 @@
 #define AMIGA_GUI_OPTIONS_H
 /* Prefs GUI control */
 void ami_gui_opts_open(void);
-BOOL ami_gui_opts_event(void);
-void ami_gui_opts_close(void);
 
 /* Web search list */
 struct List *ami_gui_opts_websearch(void);
