@@ -22,7 +22,7 @@
 struct gui_search_table;
 struct gui_window;
 
-struct gui_search_table *amiga_search_table;
+extern struct gui_search_table *amiga_search_table;
 
 /**
  * Change the displayed search status.
@@ -31,11 +31,6 @@ struct gui_search_table *amiga_search_table;
  */
 void ami_search_open(struct gui_window *gwin);
 
-/**
- * Process search events
- */
-BOOL ami_search_event(void);
-
 /**
  * Close search
  */
