@@ -35,9 +35,7 @@
 #include "netsurf/mouse.h"
 #include "netsurf/plotters.h"
 #include "netsurf/keypress.h"
-#include "desktop/plot_style.h"
 #include "desktop/browser_history.h"
-#include "desktop/tree.h"
 #include "desktop/hotlist.h"
 #include "desktop/textarea.h"
 
@@ -96,7 +94,7 @@ static float toolbar_url_scale = 1.0;
 
 static plot_font_style_t font_style_url = {
 	.family = PLOT_FONT_FAMILY_SANS_SERIF,
-	.size = 14*FONT_SIZE_SCALE,
+	.size = 14*PLOT_STYLE_SCALE,
 	.weight = 400,
 	.flags = FONTF_NONE,
 	.background = 0xffffff,
@@ -271,7 +269,7 @@ struct s_toolbar *toolbar_create(struct s_gui_win_root *owner)
 	int i;
 	struct s_toolbar *t;
 
-	LOG("owner %p", owner);
+	NSLOG(netsurf, INFO, "owner %p", owner);
 
 	assert(init == true);
 
@@ -304,7 +302,7 @@ struct s_toolbar *toolbar_create(struct s_gui_win_root *owner)
 
 	/* create the url widget: */
 	font_style_url.size =
-		toolbar_styles[t->style].font_height_pt * FONT_SIZE_SCALE;
+		toolbar_styles[t->style].font_height_pt * PLOT_STYLE_SCALE;
 
 	textarea_flags ta_flags = TEXTAREA_INTERNAL_CARET;
 	textarea_setup ta_setup;
@@ -329,8 +327,9 @@ struct s_toolbar *toolbar_create(struct s_gui_win_root *owner)
 	t->throbber.max_index = THROBBER_MAX_INDEX;
 	t->throbber.running = false;
 
-	LOG("created toolbar: %p, root: %p, textarea: %p, throbber: %p",
-            t, owner, t->url.textarea, &t->throbber);
+	NSLOG(netsurf, INFO,
+	      "created toolbar: %p, root: %p, textarea: %p, throbber: %p", t,
+	      owner, t->url.textarea, &t->throbber);
 	return( t );
 }
 
@@ -437,7 +436,7 @@ void toolbar_redraw(struct s_toolbar *tb, GRECT *clip)
 	if (rc_intersect(clip, &area)) {
 		float old_scale;
 
-		plot_set_dimensions(area_ro.g_x, area_ro.g_y, area_ro.g_w, area_ro.g_h);
+		plot_set_dimensions(&toolbar_rdrw_ctx, area_ro.g_x, area_ro.g_y, area_ro.g_w, area_ro.g_h);
 		struct rect r = {
 			.x0 = MAX(0,area.g_x - area_ro.g_x),
 			.y0 = MAX(0,area.g_y - area_ro.g_y),
@@ -460,7 +459,7 @@ toolbar_update_buttons(struct s_toolbar *tb,
                        struct browser_window *bw,
                        short button)
 {
-        LOG("tb %p", tb);
+        NSLOG(netsurf, INFO, "tb %p", tb);
 
 	struct s_tb_button * bt;
 	bool enable = false;
@@ -584,7 +583,7 @@ void toolbar_set_dimensions(struct s_toolbar *tb, GRECT *area)
 
 void toolbar_set_url(struct s_toolbar *tb, const char *text)
 {
-	LOG("tb %p", tb);
+	NSLOG(netsurf, INFO, "tb %p", tb);
 
 	textarea_set_text(tb->url.textarea, text);
 
@@ -670,7 +669,7 @@ bool toolbar_text_input(struct s_toolbar *tb, char *text)
 {
 	bool handled = true;
 
-	LOG("tb %p", tb);
+	NSLOG(netsurf, INFO, "tb %p", tb);
 
 	return(handled);
 }
@@ -759,7 +758,7 @@ void toolbar_mouse_input(struct s_toolbar *tb, short obj, short button)
 	short mx, my, mb, kstat;
 	struct gui_window * gw;
 
-        LOG("tb %p", tb);
+        NSLOG(netsurf, INFO, "tb %p", tb);
 
 
 	if (obj==TOOLBAR_AREA_URL) {
