@@ -133,6 +133,9 @@ struct beos_scaffolding {
 	int			being_destroyed;
 
 	bool			fullscreen;
+
+	/** Object under menu, or 0 if no object. */
+	struct hlcache_handle		*current_menu_object;
 };
 
 struct beos_history_window {
@@ -605,8 +608,10 @@ NSBaseView::Instantiate(BMessage *archive)
 
 	struct replicant_thread_info *info = new replicant_thread_info;
 	info->url = BString(url);
-	if (nsbeos_find_app_path(info->app) < B_OK)
+	if (nsbeos_find_app_path(info->app) < B_OK) {
+                delete info;
 		return NULL;
+        }
 	info->args[0] = info->app;
 	info->args[1] = (char *)info->url.String();
 	info->args[2] = NULL;
@@ -729,6 +734,15 @@ NSBrowserWindow::MessageReceived(BMessage *message)
 		case B_ARGV_RECEIVED:
 		case B_REFS_RECEIVED:
 		case B_UI_SETTINGS_CHANGED:
+		// NetPositive messages
+		case B_NETPOSITIVE_OPEN_URL:
+		case B_NETPOSITIVE_BACK:
+		case B_NETPOSITIVE_FORWARD:
+		case B_NETPOSITIVE_HOME:
+		case B_NETPOSITIVE_RELOAD:
+		case B_NETPOSITIVE_STOP:
+		case B_NETPOSITIVE_DOWN:
+		case B_NETPOSITIVE_UP:
 			DetachCurrentMessage();
 			nsbeos_pipe_message_top(message, this, fScaffolding);
 			break;
@@ -782,7 +796,7 @@ int32 nsbeos_replicant_main_thread(void *_arg)
 
 static void nsbeos_window_destroy_event(NSBrowserWindow *window, nsbeos_scaffolding *g, BMessage *event)
 {
-	LOG("Being Destroyed = %d", g->being_destroyed);
+	NSLOG(netsurf, INFO, "Being Destroyed = %d", g->being_destroyed);
 
 	if (--open_windows == 0)
 		nsbeos_done = true;
@@ -840,7 +854,9 @@ void nsbeos_scaffolding_dispatch_event(nsbeos_scaffolding *scaffold, BMessage *m
 	bw = nsbeos_get_browser_for_gui(scaffold->top_level);
 	bool reloadAll = false;
 
-	LOG("nsbeos_scaffolding_dispatch_event() what = 0x%08lx", message->what);
+	NSLOG(netsurf, INFO,
+	      "nsbeos_scaffolding_dispatch_event() what = 0x%08lx",
+	      message->what);
 	switch (message->what) {
 		case B_QUIT_REQUESTED:
 			nsbeos_scaffolding_destroy(scaffold);
@@ -983,7 +999,7 @@ void nsbeos_scaffolding_dispatch_event(nsbeos_scaffolding *scaffold, BMessage *m
 			browser_window_key_press(bw, NS_KEY_PASTE);
 			break;
 		case B_SELECT_ALL:
-			LOG("Selecting all text");
+			NSLOG(netsurf, INFO, "Selecting all text");
 			browser_window_key_press(bw, NS_KEY_SELECT_ALL);
 			break;
 		case B_NETPOSITIVE_BACK:
@@ -1245,6 +1261,11 @@ void nsbeos_scaffolding_dispatch_event(nsbeos_scaffolding *scaffold, BMessage *m
 		case BROWSER_OBJECT_INFO:
 			break;
 		case BROWSER_OBJECT_RELOAD:
+			if (scaffold->current_menu_object != NULL) {
+				content_invalidate_reuse_data(scaffold->current_menu_object);
+				browser_window_reload(bw, false);
+				scaffold->current_menu_object = NULL;
+			}
 			break;
 		case BROWSER_OBJECT_SAVE:
 			break;
@@ -1340,7 +1361,8 @@ void nsbeos_scaffolding_dispatch_event(nsbeos_scaffolding *scaffold, BMessage *m
 
 void nsbeos_scaffolding_destroy(nsbeos_scaffolding *scaffold)
 {
-	LOG("Being Destroyed = %d", scaffold->being_destroyed);
+	NSLOG(netsurf, INFO, "Being Destroyed = %d",
+	      scaffold->being_destroyed);
 	if (scaffold->being_destroyed) return;
 	scaffold->being_destroyed = 1;
 	nsbeos_window_destroy_event(scaffold->window, scaffold, NULL);
@@ -1425,7 +1447,7 @@ static void recursively_set_menu_items_target(BMenu *menu, BHandler *handler)
 
 void nsbeos_attach_toplevel_view(nsbeos_scaffolding *g, BView *view)
 {
-	LOG("Attaching view to scaffolding %p", g);
+	NSLOG(netsurf, INFO, "Attaching view to scaffolding %p", g);
 
 	// this is a replicant,... and it went bad
 	if (!g->window) {
@@ -1505,6 +1527,8 @@ void nsbeos_attach_toplevel_view(nsbeos_scaffolding *g, BView *view)
 
 	nsbeos_scaffolding_update_colors(g);
 
+	recursively_set_menu_items_target(g->popup_menu, view);
+
 	if (g->window) {
 		recursively_set_menu_items_target(g->menu_bar, view);
 
@@ -1557,36 +1581,34 @@ static BMenuItem *make_menu_item(const char *name, BMessage *message, bool enabl
 {
 	BMenuItem *item;
 	BString label(messages_get(name));
-	BString accel;
+	BString accelKey(name);
+	accelKey += "Accel";
+	BString accel(messages_get(accelKey));
+	if (accel == accelKey)
+		accel = "";
 	uint32 mods = 0;
 	char key = 0;
 	// try to understand accelerators
-	int32 start = label.IFindLast(" ");
-	if (start > 0 && (label.Length() - start > 1)
-		&& (label.Length() - start < 7) 
-		&& (label[start + 1] == 'F' 
-		|| !strcmp(label.String() + start + 1, "PRINT")
-		|| label[start + 1] == '\xe2'
-		|| label[start + 1] == '^')) {
-
-		label.MoveInto(accel, start + 1, label.Length());
-		// strip the trailing spaces
-		while (label[label.Length() - 1] == ' ')
-			label.Truncate(label.Length() - 1);
-
+	if (!accel.IsEmpty()) {
 		if (accel.FindFirst("\xe2\x87\x91") > -1) {
 			accel.RemoveFirst("\xe2\x87\x91");
 			mods |= B_SHIFT_KEY;
 		}
 		if (accel.FindFirst("^") > -1) {
 			accel.RemoveFirst("^");
-			mods |= B_CONTROL_KEY; // ALT!!!
+			mods |= B_CONTROL_KEY;
 		}
 		if (accel.FindFirst("PRINT") > -1) {
 			accel.RemoveFirst("PRINT");
 			//mods |= ; // ALT!!!
 			key = B_PRINT_KEY;
 		}
+
+		/* replace UTF-8 glyphs (arrows...) with API codes */
+		accel.ReplaceAll("\xE2\x86\x90", (BString()+=B_LEFT_ARROW).String());
+		accel.ReplaceAll("\xE2\x86\x92", (BString()+=B_RIGHT_ARROW).String());
+		accel.ReplaceAll("\xE2\x86\x91", (BString()+=B_UP_ARROW).String());
+
 		if (accel.Length() > 1 && accel[0] == 'F') { // Function key
 			int num;
 			if (sscanf(accel.String(), "F%d", &num) > 0) {
@@ -1703,7 +1725,8 @@ nsbeos_scaffolding *nsbeos_new_scaffolding(struct gui_window *toplevel)
 {
 	struct beos_scaffolding *g = (struct beos_scaffolding *)malloc(sizeof(*g));
 
-	LOG("Constructing a scaffold of %p for gui_window %p", g, toplevel);
+	NSLOG(netsurf, INFO,
+	      "Constructing a scaffold of %p for gui_window %p", g, toplevel);
 
 	g->top_level = toplevel;
 	g->being_destroyed = 0;
@@ -1713,6 +1736,7 @@ nsbeos_scaffolding *nsbeos_new_scaffolding(struct gui_window *toplevel)
 
 	BMessage *message;
 	BRect rect;
+	BMenuItem *item;
 
 	g->window = NULL;
 	g->menu_bar = NULL;
@@ -1760,7 +1784,6 @@ nsbeos_scaffolding *nsbeos_new_scaffolding(struct gui_window *toplevel)
 		g->window->AddChild(g->menu_bar);
 
 		BMenu *menu;
-		BMenuItem *item;
 
 		// App menu
 		//XXX: use icon item ?
@@ -2064,7 +2087,22 @@ nsbeos_scaffolding *nsbeos_new_scaffolding(struct gui_window *toplevel)
 	g->top_view->SetScaffolding(g);
 
 	// build popup menu
-	g->popup_menu = new BPopUpMenu("");
+	g->popup_menu = new BPopUpMenu("popup", false, false);
+
+#if 0
+	message = new BMessage(BROWSER_OBJECT_INFO);
+	item = make_menu_item("ObjInfo", message);
+	g->popup_menu->AddItem(item);
+
+	message = new BMessage(BROWSER_OBJECT_SAVE);
+	item = make_menu_item("ObjSave", message);
+	g->popup_menu->AddItem(item);
+	// XXX: submenu: Sprite ?
+#endif
+
+	message = new BMessage(BROWSER_OBJECT_RELOAD);
+	item = make_menu_item("ObjReload", message, true);
+	g->popup_menu->AddItem(item);
 
 
 #ifdef ENABLE_DRAGGER
@@ -2328,8 +2366,19 @@ void gui_window_set_icon(struct gui_window *_g, hlcache_handle *icon)
 }
 
 
-void nsbeos_scaffolding_popup_menu(nsbeos_scaffolding *g, BPoint where)
+void nsbeos_scaffolding_popup_menu(nsbeos_scaffolding *scaffold, struct browser_window *bw, BPoint where, BPoint screenWhere)
 {
-	g->popup_menu->Go(where);
+	struct browser_window_features cont;
+
+	browser_window_get_features(bw, (int)where.x, (int)where.y, &cont);
+
+	scaffold->current_menu_object = cont.object;
+	bool enabled = !!scaffold->current_menu_object;
+
+	for (int i = 0; scaffold->popup_menu->ItemAt(i); i++) {
+		scaffold->popup_menu->ItemAt(i)->SetEnabled(enabled);
+	}
+
+	scaffold->popup_menu->Go(screenWhere, true, false, true);
 }
 
