@@ -107,6 +107,8 @@ MENUHANDLER(delete_selected)
 
 MENUHANDLER(delete_all)
 {
+	global_history_keypress(NS_KEY_ESCAPE);
+	global_history_keypress(NS_KEY_ESCAPE);
 	global_history_keypress(NS_KEY_SELECT_ALL);
 	global_history_keypress(NS_KEY_DELETE_LEFT);
 	return TRUE;
@@ -114,12 +116,16 @@ MENUHANDLER(delete_all)
 
 MENUHANDLER(select_all)
 {
+	global_history_keypress(NS_KEY_ESCAPE);
+	global_history_keypress(NS_KEY_ESCAPE);
 	global_history_keypress(NS_KEY_SELECT_ALL);
 	return TRUE;
 }
 
 MENUHANDLER(clear_selection)
 {
+	global_history_keypress(NS_KEY_ESCAPE);
+	global_history_keypress(NS_KEY_ESCAPE);
 	global_history_keypress(NS_KEY_CLEAR_SELECTION);
 	return TRUE;
 }
@@ -343,8 +349,7 @@ static nserror nsgtk_global_history_init(void)
 		return res;
 	}
 
-	res = global_history_init(ncwin->core.cb_table,
-				  (struct core_window *)ncwin);
+	res = global_history_init((struct core_window *)ncwin);
 	if (res != NSERROR_OK) {
 		free(ncwin);
 		return res;
