@@ -170,6 +170,8 @@ MENUHANDLER(delete_selected)
 
 MENUHANDLER(select_all)
 {
+	hotlist_keypress(NS_KEY_ESCAPE);
+	hotlist_keypress(NS_KEY_ESCAPE);
 	hotlist_keypress(NS_KEY_SELECT_ALL);
 	return TRUE;
 }
@@ -361,8 +363,7 @@ static nserror nsgtk_hotlist_init(void)
 		return res;
 	}
 
-	res = hotlist_manager_init(ncwin->core.cb_table,
-			   (struct core_window *)ncwin);
+	res = hotlist_manager_init((struct core_window *)ncwin);
 	if (res != NSERROR_OK) {
 		free(ncwin);
 		return res;
