@@ -17,8 +17,10 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
- /** \file
-  * GTK printing (implementation).
+
+/**
+ * \file
+  * GTK printing implementation.
   * All the functions and structures necessary for printing( signal handlers,
   * plotters, printer) are here.
   * Most of the plotters have been copied from the gtk_plotters.c file.
@@ -71,7 +73,7 @@ static inline void nsgtk_print_set_colour(colour c)
 
 
 
-static bool gtk_print_font_paint(int x, int y, 
+static nserror gtk_print_font_paint(int x, int y, 
 		const char *string, size_t length,
 		const plot_font_style_t *fstyle)
 {
@@ -81,7 +83,7 @@ static bool gtk_print_font_paint(int x, int y,
 	PangoLayoutLine *line;
 
 	if (length == 0)
-		return true;
+		return NSERROR_OK;
 
 	desc = nsfont_style_to_description(fstyle);
 	size = (gint) ((double) pango_font_description_get_size(desc) * 
@@ -106,7 +108,7 @@ static bool gtk_print_font_paint(int x, int y,
 	g_object_unref(layout);
 	pango_font_description_free(desc);
 
-	return true;
+	return NSERROR_OK;
 }
 
 
@@ -131,10 +133,32 @@ static inline void nsgtk_print_set_dashed(void)
 	cairo_set_dash(gtk_print_current_cr, cdashes, 1, 0);
 }
 
-/** Set clipping area for subsequent plot operations. */
-static bool nsgtk_print_plot_clip(const struct rect *clip)
+/** Set cairo context line width. */
+static inline void nsgtk_set_line_width(plot_style_fixed width)
 {
-	LOG("Clipping. x0: %i ;\t y0: %i ;\t x1: %i ;\t y1: %i", clip->x0, clip->y0, clip->x1, clip->y1);	
+	if (width == 0) {
+		cairo_set_line_width(gtk_print_current_cr, 1);
+	} else {
+		cairo_set_line_width(gtk_print_current_cr,
+				plot_style_fixed_to_double(width));
+	}
+}
+
+
+/**
+ * \brief Sets a clip rectangle for subsequent plot operations.
+ *
+ * \param ctx The current redraw context.
+ * \param clip The rectangle to limit all subsequent plot
+ *              operations within.
+ * \return NSERROR_OK on success else error code.
+ */
+static nserror
+nsgtk_print_plot_clip(const struct redraw_context *ctx, const struct rect *clip)
+{
+	NSLOG(netsurf, INFO,
+	      "Clipping. x0: %i ;\t y0: %i ;\t x1: %i ;\t y1: %i", clip->x0,
+	      clip->y0, clip->x1, clip->y1);	
 	
 	/* Normalize cllipping area - to prevent overflows.
 	 * See comment in pdf_plot_fill. */
@@ -153,10 +177,30 @@ static bool nsgtk_print_plot_clip(const struct rect *clip)
 	cliprect.width = clip_x1 - clip_x0;
 	cliprect.height = clip_y1 - clip_y0;
 	
-	return true;	
+	return NSERROR_OK;
 }
 
-static bool nsgtk_print_plot_arc(int x, int y, int radius, int angle1, int angle2, const plot_style_t *style)
+
+/**
+ * Plots an arc
+ *
+ * plot an arc segment around (x,y), anticlockwise from angle1
+ *  to angle2. Angles are measured anticlockwise from
+ *  horizontal, in degrees.
+ *
+ * \param ctx The current redraw context.
+ * \param style Style controlling the arc plot.
+ * \param x The x coordinate of the arc.
+ * \param y The y coordinate of the arc.
+ * \param radius The radius of the arc.
+ * \param angle1 The start angle of the arc.
+ * \param angle2 The finish angle of the arc.
+ * \return NSERROR_OK on success else error code.
+ */
+static nserror
+nsgtk_print_plot_arc(const struct redraw_context *ctx,
+	       const plot_style_t *style,
+	       int x, int y, int radius, int angle1, int angle2)
 {
 	nsgtk_print_set_colour(style->fill_colour);
 	nsgtk_print_set_solid();
@@ -167,10 +211,26 @@ static bool nsgtk_print_plot_arc(int x, int y, int radius, int angle1, int angle
 			(angle2 + 90) * (M_PI / 180));
 	cairo_stroke(gtk_print_current_cr);
 
-	return true;
+	return NSERROR_OK;
 }
 
-static bool nsgtk_print_plot_disc(int x, int y, int radius, const plot_style_t *style)
+
+/**
+ * Plots a circle
+ *
+ * Plot a circle centered on (x,y), which is optionally filled.
+ *
+ * \param ctx The current redraw context.
+ * \param style Style controlling the circle plot.
+ * \param x x coordinate of circle centre.
+ * \param y y coordinate of circle centre.
+ * \param radius circle radius.
+ * \return NSERROR_OK on success else error code.
+ */
+static nserror
+nsgtk_print_plot_disc(const struct redraw_context *ctx,
+		const plot_style_t *style,
+		int x, int y, int radius)
 {
 	if (style->fill_type != PLOT_OP_TYPE_NONE) {
 		nsgtk_print_set_colour(style->fill_colour);
@@ -199,19 +259,31 @@ static bool nsgtk_print_plot_disc(int x, int y, int radius, const plot_style_t *
 			break;
 		}
 
-		if (style->stroke_width == 0)
-			cairo_set_line_width(gtk_print_current_cr, 1);
-		else
-			cairo_set_line_width(gtk_print_current_cr, style->stroke_width);
+		nsgtk_set_line_width(style->stroke_width);
 
 		cairo_arc(gtk_print_current_cr, x, y, radius, 0, M_PI * 2);
 
 		cairo_stroke(gtk_print_current_cr);
 	}
-	return true;
+	return NSERROR_OK;
 }
 
-static bool nsgtk_print_plot_line(int x0, int y0, int x1, int y1, const plot_style_t *style)
+
+/**
+ * Plots a line
+ *
+ * plot a line from (x0,y0) to (x1,y1). Coordinates are at
+ *  centre of line width/thickness.
+ *
+ * \param ctx The current redraw context.
+ * \param style Style controlling the line plot.
+ * \param line A rectangle defining the line to be drawn
+ * \return NSERROR_OK on success else error code.
+ */
+static nserror
+nsgtk_print_plot_line(const struct redraw_context *ctx,
+		const plot_style_t *style,
+		const struct rect *line)
 {
 	nsgtk_print_set_colour(style->stroke_colour);
 
@@ -230,42 +302,61 @@ static bool nsgtk_print_plot_line(int x0, int y0, int x1, int y1, const plot_sty
 		break;
 	}
 
-	if (style->stroke_width == 0) 
-		cairo_set_line_width(gtk_print_current_cr, 1);
-	else
-		cairo_set_line_width(gtk_print_current_cr, style->stroke_width);
+	nsgtk_set_line_width(style->stroke_width);
 
-	cairo_move_to(gtk_print_current_cr, x0 + 0.5, y0 + 0.5);
-	cairo_line_to(gtk_print_current_cr, x1 + 0.5, y1 + 0.5);
+	cairo_move_to(gtk_print_current_cr, line->x0 + 0.5, line->y0 + 0.5);
+	cairo_line_to(gtk_print_current_cr, line->x1 + 0.5, line->y1 + 0.5);
 	cairo_stroke(gtk_print_current_cr);
 
-	return true;
+	return NSERROR_OK;
 }
 
-static bool nsgtk_print_plot_rectangle(int x0, int y0, int x1, int y1, const plot_style_t *style)
+
+/**
+ * Plots a rectangle.
+ *
+ * The rectangle can be filled an outline or both controlled
+ *  by the plot style The line can be solid, dotted or
+ *  dashed. Top left corner at (x0,y0) and rectangle has given
+ *  width and height.
+ *
+ * \param ctx The current redraw context.
+ * \param style Style controlling the rectangle plot.
+ * \param rect A rectangle defining the line to be drawn
+ * \return NSERROR_OK on success else error code.
+ */
+static nserror
+nsgtk_print_plot_rectangle(const struct redraw_context *ctx,
+		     const plot_style_t *style,
+		     const struct rect *rect)
 {
-	LOG("x0: %i ;\t y0: %i ;\t x1: %i ;\t y1: %i", x0, y0, x1, y1);
+	NSLOG(netsurf, INFO, "x0: %i ;\t y0: %i ;\t x1: %i ;\t y1: %i",
+	      rect->x0, rect->y0, rect->x1, rect->y1);
 
         if (style->fill_type != PLOT_OP_TYPE_NONE) { 
+		int x0,y0,x1,y1;
 
 		nsgtk_print_set_colour(style->fill_colour);
 		nsgtk_print_set_solid();
 	
 		/* Normalize boundaries of the area - to prevent overflows.
 		 * See comment in pdf_plot_fill. */
-		x0 = min(max(x0, 0), settings->page_width);
-		y0 = min(max(y0, 0), settings->page_height);
-		x1 = min(max(x1, 0), settings->page_width);
-		y1 = min(max(y1, 0), settings->page_height);
+		x0 = min(max(rect->x0, 0), settings->page_width);
+		y0 = min(max(rect->y0, 0), settings->page_height);
+		x1 = min(max(rect->x1, 0), settings->page_width);
+		y1 = min(max(rect->y1, 0), settings->page_height);
 
 		cairo_set_line_width(gtk_print_current_cr, 0);
-		cairo_rectangle(gtk_print_current_cr, x0, y0, x1 - x0, y1 - y0);
+		cairo_rectangle(gtk_print_current_cr,
+				x0, y0,
+				x1 - x0, y1 - y0);
 		cairo_fill(gtk_print_current_cr);
 		cairo_stroke(gtk_print_current_cr);
 	}
 
         if (style->stroke_type != PLOT_OP_TYPE_NONE) { 
-                nsgtk_print_set_colour(style->stroke_colour);
+
+		nsgtk_print_set_colour(style->stroke_colour);
 
                 switch (style->stroke_type) {
                 case PLOT_OP_TYPE_SOLID: /**< Solid colour */
@@ -282,23 +373,28 @@ static bool nsgtk_print_plot_rectangle(int x0, int y0, int x1, int y1, const plo
                         break;
                 }
 
-                if (style->stroke_width == 0) 
-                        cairo_set_line_width(gtk_print_current_cr, 1);
-                else
-                        cairo_set_line_width(gtk_print_current_cr, style->stroke_width);
+		nsgtk_set_line_width(style->stroke_width);
+
+		cairo_rectangle(gtk_print_current_cr,
+				rect->x0, rect->y0,
+				rect->x1 - rect->x0, rect->y1 - rect->y0);
 
-		cairo_rectangle(gtk_print_current_cr, x0, y0, x1 - x0, y1 - y0);
 		cairo_stroke(gtk_print_current_cr);
 	}
 	
-	return true;
+	return NSERROR_OK;
 }
 
-static bool nsgtk_print_plot_polygon(const int *p, unsigned int n, const plot_style_t *style)
+
+static nserror
+nsgtk_print_plot_polygon(const struct redraw_context *ctx,
+		   const plot_style_t *style,
+		   const int *p,
+		   unsigned int n)
 {
 	unsigned int i;
 
-	LOG("Plotting polygon.");	
+	NSLOG(netsurf, INFO, "Plotting polygon.");	
 
 	nsgtk_print_set_colour(style->fill_colour);
 	nsgtk_print_set_solid();
@@ -306,28 +402,46 @@ static bool nsgtk_print_plot_polygon(const int *p, unsigned int n, const plot_st
 	cairo_set_line_width(gtk_print_current_cr, 0);
 	cairo_move_to(gtk_print_current_cr, p[0], p[1]);
 
-	LOG("Starting line at: %i\t%i", p[0], p[1]);
+	NSLOG(netsurf, INFO, "Starting line at: %i\t%i", p[0], p[1]);
 
 	for (i = 1; i != n; i++) {
 		cairo_line_to(gtk_print_current_cr, p[i * 2], p[i * 2 + 1]);
-		LOG("Drawing line to: %i\t%i", p[i * 2], p[i * 2 + 1]);
+		NSLOG(netsurf, INFO, "Drawing line to: %i\t%i", p[i * 2],
+		      p[i * 2 + 1]);
 	}
 
 	cairo_fill(gtk_print_current_cr);
 	cairo_stroke(gtk_print_current_cr);
 
-	return true;
+	return NSERROR_OK;
 }
 
 
-static bool nsgtk_print_plot_path(const float *p, unsigned int n, colour fill, 
-		float width, colour c, const float transform[6])
+/**
+ * Plots a path.
+ *
+ * Path plot consisting of cubic Bezier curves. Line and fill colour is
+ *  controlled by the plot style.
+ *
+ * \param ctx The current redraw context.
+ * \param pstyle Style controlling the path plot.
+ * \param p elements of path
+ * \param n nunber of elements on path
+ * \param transform A transform to apply to the path.
+ * \return NSERROR_OK on success else error code.
+ */
+static nserror
+nsgtk_print_plot_path(const struct redraw_context *ctx,
+		const plot_style_t *pstyle,
+		const float *p,
+		unsigned int n,
+		const float transform[6])
 {
 	/* Only the internal SVG renderer uses this plot call currently,
 	 * and the GTK version uses librsvg.  Thus, we ignore this complexity,
 	 * and just return true obliviously. */
 
-	return true;
+	return NSERROR_OK;
 }
 
 
@@ -445,9 +559,37 @@ static bool nsgtk_print_plot_pixbuf(int x, int y, int width, int height,
 }
 
 
-static bool nsgtk_print_plot_bitmap(int x, int y, int width, int height,
-		struct bitmap *bitmap, colour bg,
-		bitmap_flags_t flags)
+/**
+ * Plot a bitmap
+ *
+ * Tiled plot of a bitmap image. (x,y) gives the top left
+ * coordinate of an explicitly placed tile. From this tile the
+ * image can repeat in all four directions -- up, down, left
+ * and right -- to the extents given by the current clip
+ * rectangle.
+ *
+ * The bitmap_flags say whether to tile in the x and y
+ * directions. If not tiling in x or y directions, the single
+ * image is plotted. The width and height give the dimensions
+ * the image is to be scaled to.
+ *
+ * \param ctx The current redraw context.
+ * \param bitmap The bitmap to plot
+ * \param x The x coordinate to plot the bitmap
+ * \param y The y coordiante to plot the bitmap
+ * \param width The width of area to plot the bitmap into
+ * \param height The height of area to plot the bitmap into
+ * \param bg the background colour to alpha blend into
+ * \param flags the flags controlling the type of plot operation
+ * \return NSERROR_OK on success else error code.
+ */
+static nserror
+nsgtk_print_plot_bitmap(const struct redraw_context *ctx,
+		  struct bitmap *bitmap,
+		  int x, int y,
+		  int width, int height,
+		  colour bg,
+		  bitmap_flags_t flags)
 {
 	int doneheight = 0, donewidth = 0;
 	bool repeat_x = (flags & BITMAPF_REPEAT_X);
@@ -495,12 +637,19 @@ static bool nsgtk_print_plot_bitmap(int x, int y, int width, int height,
 	return true;
 }
 
-static bool nsgtk_print_plot_text(int x, int y, const char *text, size_t length,
-		const plot_font_style_t *fstyle)
+
+static nserror
+nsgtk_print_plot_text(const struct redraw_context *ctx,
+		const struct plot_font_style *fstyle,
+		int x,
+		int y,
+		const char *text,
+		size_t length)
 {
 	return gtk_print_font_paint(x, y, text, length, fstyle);
 }
 
+
 /** GTK print plotter table */
 static const struct plotter_table nsgtk_print_plotters = {
 	.clip = nsgtk_print_plot_clip,
@@ -549,7 +698,7 @@ void gtk_print_signal_begin_print (GtkPrintOperation *operation,
 	int page_number;	
 	double height_on_page, height_to_print;
 	
-	LOG("Begin print");
+	NSLOG(netsurf, INFO, "Begin print");
 	
 	settings = user_data;
 		
@@ -568,7 +717,11 @@ void gtk_print_signal_begin_print (GtkPrintOperation *operation,
 		
 	} else {
 
-		LOG("page_width: %f ;page_height: %f; content height: %lf", settings->page_width, settings->page_height, height_to_print);
+		NSLOG(netsurf, INFO,
+		      "page_width: %f ;page_height: %f; content height: %lf",
+		      settings->page_width,
+		      settings->page_height,
+		      height_to_print);
 	
 		height_on_page = settings->page_height;
 		height_on_page = height_on_page - 
@@ -592,7 +745,7 @@ void gtk_print_signal_begin_print (GtkPrintOperation *operation,
 void gtk_print_signal_draw_page(GtkPrintOperation *operation,
 		GtkPrintContext *context, gint page_nr, gpointer user_data)
 {
-	LOG("Draw Page");
+	NSLOG(netsurf, INFO, "Draw Page");
 	gtk_print_current_cr = gtk_print_context_get_cairo_context(context);
 	print_draw_next_page(&gtk_printer, settings);
 }
@@ -604,7 +757,7 @@ void gtk_print_signal_draw_page(GtkPrintOperation *operation,
 void gtk_print_signal_end_print(GtkPrintOperation *operation,
 		GtkPrintContext *context, gpointer user_data)
 {
-	LOG("End print");	
+	NSLOG(netsurf, INFO, "End print");	
 	print_cleanup(content_to_print, &gtk_printer, user_data);
 }
 
