@@ -673,15 +673,22 @@ nsgtk_toolbar_customisation_save(struct nsgtk_toolbar *tb)
 	for (location = BACK_BUTTON;
 	     location < PLACEHOLDER_BUTTON;
 	     location++) {
+		int written;
 		itemid = itemid_from_location(tb, location);
 		if (itemid == PLACEHOLDER_BUTTON) {
 			/* no more filled locations */
 			break;
 		}
-		start += snprintf(start,
+		written = snprintf(start,
 				orderlen - (start - order),
 				"%s/",
 				tb->items[itemid].name);
+		if ((written < 0) ||
+		    (written >= orderlen - (start - order))) {
+			free(order);
+			return NSERROR_UNKNOWN;
+		}
+		start += written;
 
 		if ((start - order) >= orderlen) {
 			break;
@@ -2846,17 +2853,16 @@ debugboxtree_button_clicked_cb(GtkWidget *widget, gpointer data)
 		nsgtk_warning("Error saving box tree dump.",
 			      "Unable to open file for writing.");
 		unlink(fname);
-		return TRUE;
+
+	} else {
+		bw = tb->get_bw(tb->get_ctx);
+
+		browser_window_debug_dump(bw, f, CONTENT_DEBUG_RENDER);
+
+		fclose(f);
+
+		nsgtk_viewfile("Box Tree Debug", "boxtree", fname);
 	}
-
-	bw = tb->get_bw(tb->get_ctx);
-
-	browser_window_debug_dump(bw, f, CONTENT_DEBUG_RENDER);
-
-	fclose(f);
-
-	nsgtk_viewfile("Box Tree Debug", "boxtree", fname);
-
 	g_free(fname);
 
 	return TRUE;
@@ -2891,17 +2897,16 @@ debugdomtree_button_clicked_cb(GtkWidget *widget, gpointer data)
 		nsgtk_warning("Error saving box tree dump.",
 			      "Unable to open file for writing.");
 		unlink(fname);
-		return TRUE;
+
+	} else {
+		bw = tb->get_bw(tb->get_ctx);
+
+		browser_window_debug_dump(bw, f, CONTENT_DEBUG_DOM);
+
+		fclose(f);
+
+		nsgtk_viewfile("DOM Tree Debug", "domtree", fname);
 	}
-
-	bw = tb->get_bw(tb->get_ctx);
-
-	browser_window_debug_dump(bw, f, CONTENT_DEBUG_DOM);
-
-	fclose(f);
-
-	nsgtk_viewfile("DOM Tree Debug", "domtree", fname);
-
 	g_free(fname);
 
 	return TRUE;
@@ -3066,7 +3071,7 @@ contents_button_clicked_cb(GtkWidget *widget, gpointer data)
 	struct nsgtk_toolbar *tb = (struct nsgtk_toolbar *)data;
 	nserror res;
 
-	res = toolbar_navigate_to_url(tb, "http://www.netsurf-browser.org/documentation/");
+	res = toolbar_navigate_to_url(tb, "https://www.netsurf-browser.org/documentation/");
 	if (res != NSERROR_OK) {
 		nsgtk_warning(messages_get_errorcode(res), 0);
 	}
@@ -3087,7 +3092,7 @@ guide_button_clicked_cb(GtkWidget *widget, gpointer data)
 	struct nsgtk_toolbar *tb = (struct nsgtk_toolbar *)data;
 	nserror res;
 
-	res = toolbar_navigate_to_url(tb, "http://www.netsurf-browser.org/documentation/guide");
+	res = toolbar_navigate_to_url(tb, "https://www.netsurf-browser.org/documentation/guide");
 	if (res != NSERROR_OK) {
 		nsgtk_warning(messages_get_errorcode(res), 0);
 	}
@@ -3109,7 +3114,7 @@ info_button_clicked_cb(GtkWidget *widget, gpointer data)
 	struct nsgtk_toolbar *tb = (struct nsgtk_toolbar *)data;
 	nserror res;
 
-	res = toolbar_navigate_to_url(tb, "http://www.netsurf-browser.org/documentation/info");
+	res = toolbar_navigate_to_url(tb, "https://www.netsurf-browser.org/documentation/info");
 	if (res != NSERROR_OK) {
 		nsgtk_warning(messages_get_errorcode(res), 0);
 	}
