@@ -27,7 +27,9 @@ nserror gui_401login_open(struct nsurl *url,
 			  const char *realm,
 			  const char *username,
 			  const char *password,
-			  nserror (*cb)(const char *username,
+			  nserror (*cb)(struct nsurl *url,
+					const char *realm,
+					const char *username,
 					const char *password,
 					void *pw),
 			  void *cbpw);
