@@ -423,6 +423,7 @@ hotlist_menu_select(wimp_w w,
  */
 static nserror ro_hotlist_init(void)
 {
+	os_error *error;
 	struct ro_hotlist_window *ncwin;
 	nserror res;
 	static const struct ns_menu hotlist_menu_def = {
@@ -472,7 +473,14 @@ static nserror ro_hotlist_init(void)
 	}
 
 	/* create window from template */
-	ncwin->core.wh = wimp_create_window(dialog_hotlist_template);
+	error = xwimp_create_window(dialog_hotlist_template, &ncwin->core.wh);
+	if (error) {
+		NSLOG(netsurf, INFO, "xwimp_create_window: 0x%x: %s",
+		      error->errnum, error->errmess);
+		ro_warn_user("WimpError", error->errmess);
+		free(ncwin);
+		return NSERROR_NOMEM;
+	}
 
 	ro_gui_set_window_title(ncwin->core.wh, messages_get("Hotlist"));
 
@@ -496,8 +504,7 @@ static nserror ro_hotlist_init(void)
 		return res;
 	}
 
-	res = hotlist_manager_init(ncwin->core.cb_table,
-			   (struct core_window *)ncwin);
+	res = hotlist_manager_init((struct core_window *)ncwin);
 	if (res != NSERROR_OK) {
 		free(ncwin);
 		return res;
