@@ -156,7 +156,9 @@ bool ro_gui_wimp_event_memorise(wimp_w w)
 					ro_gui_get_icon_string(window->w, event->i));
 				if (!event->previous_value.textual) {
 					error = true;
-					LOG("Unable to store state for icon %i", event->i);
+					NSLOG(netsurf, INFO,
+					      "Unable to store state for icon %i",
+					      event->i);
 				}
 				break;
 			case EVENT_CHECKBOX:
@@ -267,7 +269,10 @@ bool ro_gui_wimp_event_transfer(wimp_w from, wimp_w to)
 	struct event_window	*window;
 	int			h;
 
-	LOG("Transferring all events from window 0x%x to window 0x%x", (unsigned int)from, (unsigned int)to);
+	NSLOG(netsurf, INFO,
+	      "Transferring all events from window 0x%x to window 0x%x",
+	      (unsigned int)from,
+	      (unsigned int)to);
 
 	window = ro_gui_wimp_event_remove_window(from);
 	if (window == NULL || window->w != from)
@@ -293,7 +298,8 @@ void ro_gui_wimp_event_finalise(wimp_w w)
 	struct event_window *window;
 	struct icon_event *event;
 
-	LOG("Removing all events for window 0x%x", (unsigned int)w);
+	NSLOG(netsurf, INFO, "Removing all events for window 0x%x",
+	      (unsigned int)w);
 	window = ro_gui_wimp_event_remove_window(w);
 	if (!window)
 		return;
@@ -330,7 +336,8 @@ void ro_gui_wimp_event_deregister(wimp_w w, wimp_i i)
 	struct event_window	*window;
 	struct icon_event	*event, *parent, *child;
 
-	LOG("Removing all events for window 0x%x, icon %d", (unsigned int)w, (int)i);
+	NSLOG(netsurf, INFO, "Removing all events for window 0x%x, icon %d",
+	      (unsigned int)w, (int)i);
 	window = ro_gui_wimp_event_get_window(w);
 	if (!window)
 		return;
@@ -344,7 +351,8 @@ void ro_gui_wimp_event_deregister(wimp_w w, wimp_i i)
 		child = event->next;
 
 		if (event->i == i) {
-			LOG("Removing event 0x%x", (unsigned int)event);
+			NSLOG(netsurf, INFO, "Removing event 0x%x",
+			      (unsigned int)event);
 
 			if (parent == NULL)
 				window->first = child;
@@ -576,7 +584,8 @@ bool ro_gui_wimp_event_menu_selection(wimp_w w, wimp_i i, wimp_menu *menu,
 	ic.i = event->data.menu_gright.field;
 	error = xwimp_get_icon_state(&ic);
 	if (error) {
-		LOG("xwimp_get_icon_state: 0x%x: %s", error->errnum, error->errmess);
+		NSLOG(netsurf, INFO, "xwimp_get_icon_state: 0x%x: %s",
+		      error->errnum, error->errmess);
 		ro_warn_user("WimpError", error->errmess);
 		return false;
 	}
@@ -586,7 +595,8 @@ bool ro_gui_wimp_event_menu_selection(wimp_w w, wimp_i i, wimp_menu *menu,
 		return prepared;
 	error = xwimp_get_caret_position(&caret);
 	if (error) {
-		LOG("xwimp_get_caret_position: 0x%x: %s", error->errnum, error->errmess);
+		NSLOG(netsurf, INFO, "xwimp_get_caret_position: 0x%x: %s",
+		      error->errnum, error->errmess);
 		ro_warn_user("WimpError", error->errmess);
 		return false;
 	}
@@ -594,7 +604,10 @@ bool ro_gui_wimp_event_menu_selection(wimp_w w, wimp_i i, wimp_menu *menu,
 		error = xwimp_set_caret_position(window->w, event->data.menu_gright.field,
 				-1, -1, -1, strlen(menu_entry->data.indirected_text.text));
 		if (error) {
-			LOG("xwimp_set_caret_position: 0x%x: %s", error->errnum, error->errmess);
+			NSLOG(netsurf, INFO,
+			      "xwimp_set_caret_position: 0x%x: %s",
+			      error->errnum,
+			      error->errmess);
 			ro_warn_user("WimpError", error->errmess);
 		}
 	}
@@ -666,7 +679,7 @@ bool ro_gui_wimp_event_mouse_click(wimp_pointer *pointer)
 			for (search = window->first; search; search = search->next)
 				if (search->i == event->data.linked_icon) break;
 			if (!search) {
-				LOG("Incorrect reference.");
+				NSLOG(netsurf, INFO, "Incorrect reference.");
 				return false;
 			}
 			stepping = search->data.numeric_field.stepping;
@@ -703,13 +716,19 @@ bool ro_gui_wimp_event_mouse_click(wimp_pointer *pointer)
 				open.w = pointer->w;
 				error = xwimp_get_window_state(&open);
 				if (error) {
-					LOG("xwimp_get_window_state: 0x%x: %s", error->errnum, error->errmess);
+					NSLOG(netsurf, INFO,
+					      "xwimp_get_window_state: 0x%x: %s",
+					      error->errnum,
+					      error->errmess);
 					ro_warn_user("WimpError", error->errmess);
 					return false;
 				}
 				error = xwimp_get_caret_position(&caret);
 				if (error) {
-					LOG("xwimp_get_caret_position: 0x%x: %s", error->errnum, error->errmess);
+					NSLOG(netsurf, INFO,
+					      "xwimp_get_caret_position: 0x%x: %s",
+					      error->errnum,
+					      error->errmess);
 					ro_warn_user("WimpError", error->errmess);
 					return false;
 				}
@@ -718,7 +737,10 @@ bool ro_gui_wimp_event_mouse_click(wimp_pointer *pointer)
 				ro_gui_menu_destroy();
 				error = xwimp_open_window(PTR_WIMP_OPEN(&open));
 				if (error) {
-					LOG("xwimp_open_window: 0x%x: %s", error->errnum, error->errmess);
+					NSLOG(netsurf, INFO,
+					      "xwimp_open_window: 0x%x: %s",
+					      error->errnum,
+					      error->errmess);
 					ro_warn_user("WimpError", error->errmess);
 					return false;
 				}
@@ -728,7 +750,10 @@ bool ro_gui_wimp_event_mouse_click(wimp_pointer *pointer)
 							caret.pos.x, caret.pos.y,
 							-1, caret.index);
 					if (error) {
-						LOG("xwimp_set_caret_position: 0x%x: %s", error->errnum, error->errmess);
+						NSLOG(netsurf, INFO,
+						      "xwimp_set_caret_position: 0x%x: %s",
+						      error->errnum,
+						      error->errmess);
 						ro_warn_user("WimpError", error->errmess);
 					}
 				}
@@ -796,7 +821,8 @@ void ro_gui_wimp_event_prepare_gright_menu(wimp_w w, struct icon_event *event)
 	ic.i = event->data.menu_gright.field;
 	error = xwimp_get_icon_state(&ic);
 	if (error) {
-		LOG("xwimp_get_icon_state: 0x%x: %s", error->errnum, error->errmess);
+		NSLOG(netsurf, INFO, "xwimp_get_icon_state: 0x%x: %s",
+		      error->errnum, error->errmess);
 		ro_warn_user("WimpError", error->errmess);
 		return;
 	}
@@ -903,7 +929,8 @@ bool ro_gui_wimp_event_keypress(wimp_key *key)
 	 */
 	error = xosbyte1(osbyte_ALPHABET_NUMBER, 127, 0, &t_alphabet);
 	if (error) {
-		LOG("failed reading alphabet: 0x%x: %s", error->errnum, error->errmess);
+		NSLOG(netsurf, INFO, "failed reading alphabet: 0x%x: %s",
+		      error->errnum, error->errmess);
 		/* prevent any corruption of ucstable */
 		t_alphabet = alphabet;
 	}
@@ -917,7 +944,10 @@ bool ro_gui_wimp_event_keypress(wimp_key *key)
 		error = xserviceinternational_get_ucs_conversion_table(
 						alphabet, &unclaimed, &ostable);
 		if (error != NULL) {
-			LOG("failed reading UCS conversion table: 0x%x: %s", error->errnum, error->errmess);
+			NSLOG(netsurf, INFO,
+			      "failed reading UCS conversion table: 0x%x: %s",
+			      error->errnum,
+			      error->errmess);
 			/* Try using our own table instead */
 			ucstable = ucstable_from_alphabet(alphabet);
 		} else if (unclaimed) {
@@ -974,7 +1004,9 @@ bool ro_gui_wimp_event_keypress(wimp_key *key)
 					/* If this ever happens,
 					 * RISC OS' UTF8 keyboard
 					 * drivers are broken */
-					LOG("unexpected UTF8 start"" byte %x (ignoring)", c);
+					NSLOG(netsurf, INFO,
+					      "unexpected UTF8 start"" byte %x (ignoring)",
+					      c);
 					return true;
 				}
 				/* Anything else is ASCII, so just
@@ -985,7 +1017,9 @@ bool ro_gui_wimp_event_keypress(wimp_key *key)
 					/* If this ever happens,
 					 * RISC OS' UTF8 keyboard
 					 * drivers are broken */
-					LOG("unexpected keycode: ""%x (ignoring)", c);
+					NSLOG(netsurf, INFO,
+					      "unexpected keycode: ""%x (ignoring)",
+					      c);
 					return true;
 				}
 
@@ -1074,7 +1108,8 @@ bool ro_gui_wimp_event_close_window(wimp_w w)
 {
 	struct event_window *window;
 
-	LOG("Close event received for window 0x%x", (unsigned int)w);
+	NSLOG(netsurf, INFO, "Close event received for window 0x%x",
+	      (unsigned int)w);
 	if (w == ro_gui_wimp_event_submenu)
 		ro_gui_wimp_event_submenu = 0;
 	window = ro_gui_wimp_event_find_window(w);
@@ -1615,7 +1650,8 @@ struct event_window *ro_gui_wimp_event_get_window(wimp_w w)
 	if (window)
 		return window;
 
-	LOG("Creating structure for window 0x%x", (unsigned int)w);
+	NSLOG(netsurf, INFO, "Creating structure for window 0x%x",
+	      (unsigned int)w);
 	window = calloc(1, sizeof(struct event_window));
 	if (!window)
 		return NULL;
