@@ -33,12 +33,32 @@
 #include "utils/log.h"
 #include "utils/nsoption.h"
 
+#ifndef TESTROOT
+#define TESTROOT "/tmp"
+#endif
+
 const char *test_choices_path = "test/data/Choices";
 const char *test_choices_short_path = "test/data/Choices-short";
 const char *test_choices_all_path = "test/data/Choices-all";
 const char *test_choices_full_path = "test/data/Choices-full";
 const char *test_choices_missing_path = "test/data/Choices-missing";
 
+/* Stubs */
+nserror nslog_set_filter_by_options() { return NSERROR_OK; }
+
+/**
+ * generate test output filenames
+ */
+static char *testnam(char *out)
+{
+	static int count = 0;
+	static char name[64];
+	int pid;
+	pid=getpid();
+	snprintf(name, 64, TESTROOT"/nsoptiontest%d%d", pid, count);
+	count++;
+	return name;
+}
 
 static nserror gui_options_init_defaults(struct nsoption_s *defaults)
 {
@@ -160,7 +180,7 @@ START_TEST(nsoption_session_test)
 	nsoption_set_colour(sys_colour_ActiveBorder, 0x00d0000d);
 
 	/* write options out */
-	outnam = tmpnam(NULL);
+	outnam = testnam(NULL);
 	res = nsoption_write(outnam, NULL, NULL);
 	ck_assert_int_eq(res, NSERROR_OK);
 
@@ -227,7 +247,7 @@ struct format_test_vec_s format_test_vec[] = {
 	},
 	{
 		NSOPTION_sys_colour_ActiveBorder,
-		"<tr><th>sys_colour_ActiveBorder</th><td>colour</td><td>default</td><td><span style=\"background-color: #d3d3d3; color: #000000; font-family:Monospace; \">#D3D3D3</span></td></tr>",
+		"<tr><th>sys_colour_ActiveBorder</th><td>colour</td><td>default</td><td><span style=\"font-family:Monospace;\">#D3D3D3</span> <span style=\"background-color: #d3d3d3; border: 1px solid #000000; display: inline-block; width: 1em; height: 1em;\"></span></td></tr>",
 		"sys_colour_ActiveBorder:d3d3d3"
 	},
 };
@@ -300,7 +320,7 @@ START_TEST(nsoption_dump_test)
 	res = nsoption_read(test_choices_path, NULL);
 	ck_assert_int_eq(res, NSERROR_OK);
 
-	outnam = tmpnam(NULL);
+	outnam = testnam(NULL);
 
 	fp = fopen(outnam, "w");
 	res = nsoption_dump(fp, NULL);
@@ -325,7 +345,7 @@ START_TEST(nsoption_write_test)
 	res = nsoption_read(test_choices_path, NULL);
 	ck_assert_int_eq(res, NSERROR_OK);
 
-	outnam = tmpnam(NULL);
+	outnam = testnam(NULL);
 
 	res = nsoption_write(outnam, NULL, NULL);
 	ck_assert_int_eq(res, NSERROR_OK);
