@@ -167,6 +167,7 @@ CORESTRING_LWC_VALUE(application_octet_stream, "application/octet-stream");
 CORESTRING_LWC_VALUE(image_gif, "image/gif");
 CORESTRING_LWC_VALUE(image_png, "image/png");
 CORESTRING_LWC_VALUE(image_jpeg, "image/jpeg");
+CORESTRING_LWC_VALUE(image_jxl, "image/jxl");
 CORESTRING_LWC_VALUE(image_bmp, "image/bmp");
 CORESTRING_LWC_VALUE(image_vnd_microsoft_icon, "image/vnd.microsoft.icon");
 CORESTRING_LWC_VALUE(image_webp, "image/webp");
@@ -295,6 +296,7 @@ CORESTRING_DOM_STRING(rect);
 CORESTRING_DOM_STRING(rel);
 CORESTRING_DOM_STRING(reset);
 CORESTRING_DOM_STRING(resize);
+CORESTRING_DOM_STRING(reversed);
 CORESTRING_DOM_STRING(rows);
 CORESTRING_DOM_STRING(rowspan);
 CORESTRING_DOM_STRING(scroll);
@@ -309,6 +311,7 @@ CORESTRING_DOM_STRING(size);
 CORESTRING_DOM_STRING(sizes);
 CORESTRING_DOM_STRING(src);
 CORESTRING_DOM_STRING(stalled);
+CORESTRING_DOM_STRING(start);
 CORESTRING_DOM_STRING(storage);
 CORESTRING_DOM_STRING(style);
 CORESTRING_DOM_STRING(submit);
