@@ -27,8 +27,10 @@
 
 #include "utils/errors.h"
 
+#include "utils/http/cache-control.h"
 #include "utils/http/content-disposition.h"
 #include "utils/http/content-type.h"
+#include "utils/http/strict-transport-security.h"
 #include "utils/http/www-authenticate.h"
 
 #endif
