@@ -23,20 +23,11 @@
  * \todo Many of these functions and macros should have their own headers.
  */
 
-#ifndef _NETSURF_UTILS_UTILS_H_
-#define _NETSURF_UTILS_UTILS_H_
+#ifndef NETSURF_UTILS_UTILS_H
+#define NETSURF_UTILS_UTILS_H
 
-#include <inttypes.h>
 #include <stdbool.h>
 
-/** Rectangle coordinates */
-struct rect {
-	int x0, y0; /**< Top left */
-	int x1, y1; /**< Bottom right */
-};
-
-struct dirent;
-
 #ifndef NOF_ELEMENTS
 #define NOF_ELEMENTS(array) (sizeof(array)/sizeof(*(array)))
 #endif
@@ -58,31 +49,17 @@ struct dirent;
 #ifndef max
 #define max(x,y) (((x)>(y))?(x):(y))
 #endif
+
+#ifndef clamp
+#define clamp(x,low,high) (min(max((x),(low)),(high)))
+#endif
 #endif
 
-#ifndef PRIxPTR
-#define PRIxPTR "x"
-#endif
-
-#ifndef PRId64
-#define PRId64 "lld"
-#endif
-
-/* Windows does not have sizet formating codes or POSIX mkdir so work
- * around that
- */
+/* Windows does not have POSIX mkdir so work around that */
 #if defined(_WIN32)
-/** windows printf formatting for size_t type */
-#define PRIsizet "Iu"
-/** windows printf formatting for ssize_t type */
-#define PRIssizet "Id"
 /** windows mkdir function */
 #define nsmkdir(dir, mode) mkdir((dir))
 #else
-/** c99 standard printf formatting for size_t type */
-#define PRIsizet "zu"
-/** c99 standard printf formatting for ssize_t type */
-#define PRIssizet "zd"
 /** POSIX mkdir function */
 #define nsmkdir(dir, mode) mkdir((dir), (mode))
 #endif
@@ -116,4 +93,31 @@ struct dirent;
  */
 bool is_dir(const char *path);
 
+/**
+ * switch fall through
+ */
+#if defined __cplusplus && defined __has_cpp_attribute
+    #if __has_cpp_attribute(fallthrough) && __cplusplus >= __has_cpp_attribute(fallthrough)
+        #define fallthrough [[fallthrough]]
+    #elif __has_cpp_attribute(gnu::fallthrough) && __STDC_VERSION__ >= __has_cpp_attribute(gnu::fallthrough)
+        #define fallthrough [[gnu::fallthrough]]
+    #elif __has_cpp_attribute(clang::fallthrough) && __STDC_VERSION__ >= __has_cpp_attribute(clang::fallthrough)
+        #define fallthrough [[clang::fallthrough]]
+    #endif
+#elif defined __STDC_VERSION__ && defined __has_c_attribute
+    #if __has_c_attribute(fallthrough) && __STDC_VERSION__ >= __has_c_attribute(fallthrough)
+        #define fallthrough [[fallthrough]]
+    #endif
+#endif
+#if !defined fallthrough && defined __has_attribute
+    #if __has_attribute(__fallthrough__)
+        #define fallthrough __attribute__((__fallthrough__))
+    #endif
+#endif
+#if !defined fallthrough
+/*  early gcc and clang have no implicit fallthrough warning */
+    #define fallthrough do {} while(0)
+#endif
+
+
 #endif
